# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
# Copyright(C) 2012 François Revol
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.video import BaseVideo
from weboob.capabilities.image import Thumbnail
from weboob.capabilities.collection import Collection

from weboob.exceptions import ParseError
from weboob.browser.elements import ItemElement, ListElement, method, DictElement
from weboob.browser.pages import HTMLPage, pagination, JsonPage, XMLPage
from weboob.browser.filters.standard import Regexp, Env, CleanText, DateTime, Duration, Field, BrowserURL
from weboob.browser.filters.html import Attr, Link, XPath
from weboob.browser.filters.json import Dict

import re


class VimeoDuration(Duration):
    _regexp = re.compile(r'(?P<ss>\d+)')
    kwargs = {'seconds': 'ss'}


class ListPage(HTMLPage):
    def get_token(self):
        return Regexp(CleanText('//script'), '"jwt":"(.*)","url"', default=None)(self.doc)

    @pagination
    @method
    class iter_videos(ListElement):
        item_xpath = '//div[@id="browse_content"]/ol/li'
        next_page = Link(u'//a[text()="Next"]')

        class item(ItemElement):
            klass = BaseVideo
            obj_id = Regexp(Attr('.', 'id'), 'clip_(.*)')
            obj_title = Attr('./a', 'title')

            def obj_thumbnail(self):
                thumbnail = Thumbnail(self.xpath('./a/img')[0].attrib['src'])
                thumbnail.url = thumbnail.id
                return thumbnail


class APIPage(JsonPage):
    @pagination
    @method
    class iter_videos(DictElement):
        item_xpath = 'data'

        def parse(self, el):
            self.env['next_page'] = Regexp(Dict('paging/next'), 'page=(\d*)', default=None)(el)

        def next_page(self):
            if Env('next_page')(self) is not None:
                return BrowserURL('api_page', page=int(Env('next_page')(self)), category=Env('category'))(self)

        class item(ItemElement):
            klass = BaseVideo

            obj_id = Regexp(Dict('clip/uri'), '/videos/(.*)')
            obj_title = Dict('clip/name')

            def obj_thumbnail(self):
                thumbnail = Thumbnail(Dict('clip/pictures/sizes/0/link')(self))
                thumbnail.url = thumbnail.id
                return thumbnail


class VideoJsonPage(JsonPage):
    @method
    class fill_url(ItemElement):
        klass = BaseVideo

        def obj_url(self):
            data = self.el

            if not data['request']['files']:
                raise ParseError('Unable to detect any stream method for id: %r (available: %s)'
                                 % (int(Field('id')(self)),
                                    data['request']['files'].keys()))

            # Choosen method is not available, we choose an other one
            method = self.obj._method
            if method not in data['request']['files']:
                method = data['request']['files'].keys()[0]

            streams = data['request']['files'][method]
            if not streams:
                raise ValueError('There is no url available for id: %r' % (int(Field('id')(self))))

            # stream is single for hls, just return the url
            stream = streams['url'] if method == 'hls' else None

            # ...but a list for progressive
            # we assume the list is sorted by quality with best first
            if not stream:
                quality = self.obj._quality
                stream = streams[quality]['url'] if quality < len(streams) else streams[0]['url']

            return stream

        obj_ext = Regexp(Field('url'), '.*\.(.*)\?.*$', '\\1')


class CategoriesPage(HTMLPage):
    @method
    class iter_categories(ListElement):
        item_xpath = '//div[@class="category_grid"]/div/a'

        class item(ItemElement):
            klass = Collection

            obj_id = CleanText('./@href')
            obj_title = CleanText('./div/div/p')

            def obj_split_path(self):
                split_path = ['vimeo-categories']
                category = CleanText('./@href', replace=[('/categories/', '')])(self)
                split_path.append(category)
                return split_path


class VimeoItem(ItemElement):
    klass = BaseVideo

    obj_id = CleanText('./@id')
    obj_title = CleanText('./title')
    obj_description = CleanText('./description')
    obj_author = CleanText('./owner/@display_name')
    obj_date = DateTime(CleanText('./upload_date'))
    obj__is_hd = CleanText('./@is_hd')
    obj_duration = VimeoDuration(CleanText('./duration'))

    def obj_thumbnail(self):
        t = CleanText('./thumbnails/thumbnail[1]', default='')(self)
        if t:
            thumbnail = Thumbnail(t)
            thumbnail.url = thumbnail.id
            return thumbnail


class XMLAPIPage(XMLPage):
    @method
    class iter_videos(ListElement):
        item_xpath = '//video'

        class item(VimeoItem):
            pass

    @method
    class fill_video_infos(VimeoItem):
        def __init__(self, *args, **kwargs):
            super(VimeoItem, self).__init__(*args, **kwargs)
            self.el = XPath('/rsp/video')(self.el)[0]

    @pagination
    @method
    class iter_channels(ListElement):
        item_xpath = '//channel'

        class item(ItemElement):
            klass = Collection

            obj_title = CleanText('./name')
            obj_id = CleanText('./@id')

            def obj_split_path(self):
                split_path = ['vimeo-channels']
                split_path.append(Regexp(CleanText('./url'),
                                         'http://vimeo.com/channels/(.*)/?')(self))
                return split_path



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.tools.test import BackendTest, SkipTest


class FeedlyTest(BackendTest):
    MODULE = 'feedly'

    def test_login(self):
        if self.backend.browser.username:
            l1 = list(self.backend.iter_threads())
            assert len(l1)
            thread = self.backend.get_thread(l1[0].id)
            assert len(thread.root.content)
        else:
            raise SkipTest("User credentials not defined")

    def test_feedly(self):
        self.backend.browser.username = None
        l1 = list(self.backend.iter_resources(None, ['Technologie', 'Korben']))
        assert len(l1)
        thread = self.backend.get_thread(l1[0].id)
        assert len(thread.root.content)



# -*- coding: utf-8 -*-

# Copyright(C) 2012  Fourcot Florent
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import time
from io import BytesIO

from weboob.deprecated.browser import Browser, BrowserIncorrectPassword
from .pages import HomePage, LoginPage, HistoryPage, PdfPage
from weboob.capabilities.bill import Detail
from weboob.capabilities.base import NotAvailable


__all__ = ['Leclercmobile']


class Leclercmobile(Browser):
    DOMAIN = 'www.securelmobile.fr'
    PROTOCOL = 'https'
    ENCODING = 'utf-8'
    PAGES = {'.*pgeWERL008_Login.aspx.*':       LoginPage,
             '.*EspaceClient/pgeWERL013_Accueil.aspx':     HomePage,
             '.*pgeWERL009_ReleveConso.aspx.*': HistoryPage,
             '.*ReleveConso.ashx.*': PdfPage
             }
    accueil = "/EspaceClient/pgeWERL013_Accueil.aspx"
    login = "/EspaceClient/pgeWERL008_Login.aspx"
    conso = "/EspaceClient/pgeWERL009_ReleveConso.aspx"
    bills = '/EspaceClient/pgeWERL015_RecupReleveConso.aspx?m=-'

    def __init__(self, *args, **kwargs):
        Browser.__init__(self, *args, **kwargs)

    def home(self):
        self.location(self.accueil)

    def is_logged(self):
        return not self.is_on_page(LoginPage)

    def login(self):
        assert isinstance(self.username, basestring)
        assert isinstance(self.password, basestring)
        assert self.username.isdigit()

        if not self.is_on_page(LoginPage):
            self.location(self.login)

        form = self.page.login(self.username, self.password)

        # Site display a javascript popup to wait
        while self.page.iswait():
            # In this popup can be an error displayed
            if self.page.iserror():
                raise BrowserIncorrectPassword()
            time.sleep(1)
            self.page.next(self.username, form)

        # The last document contain a redirect url in the javascript
        self.location(self.page.getredirect())

        if self.is_on_page(LoginPage):
            raise BrowserIncorrectPassword()

    def viewing_html(self):
        # To prevent unknown mimetypes sent by server, we assume we
        # are always on a HTML document.
        return True

    def get_subscription_list(self):
        if not self.is_on_page(HomePage):
            self.location(self.acceuil)

        return self.page.get_list()

    def get_subscription(self, id):
        assert isinstance(id, basestring)

        if not self.is_on_page(HomePage):
            self.location(self.accueil)

        l = self.page.get_list()
        for a in l:
            if a.id == id:
                return a

        return None

    def get_history(self):
        if not self.is_on_page(HistoryPage):
            self.location(self.conso)
        maxid = self.page.getmaxid()

        for i in range(maxid + 1):
            response = self.openurl(self.bills + str(i))
            mimetype = response.info().get('Content-Type', '').split(';')[0]
            if mimetype == "application/pdf":
                pdf = PdfPage(BytesIO(response.read()))
                for call in pdf.get_calls():
                    call.label = call.label.strip()
                    yield call

    def get_details(self):
        if not self.is_on_page(HistoryPage):
            self.location(self.conso)
        response = self.openurl(self.bills + "0")
        mimetype = response.info().get('Content-Type', '').split(';')[0]
        if mimetype == "application/pdf":
            pdf = PdfPage(BytesIO(response.read()))
            for detail in pdf.get_details():
                yield detail

    def iter_documents(self, parentid):
        if not self.is_on_page(HistoryPage):
            self.location(self.conso)
        return self.page.date_bills(parentid)

    def get_document(self, id):
        assert isinstance(id, basestring)
        if not self.is_on_page(HistoryPage):
            self.location(self.conso)
        parentid = id[0:10]
        l = self.page.date_bills(parentid)
        for a in l:
            if a.id == id:
                return a

    def get_balance(self):
        if not self.is_on_page(HistoryPage):
            self.location(self.conso)
        detail = Detail()
        detail.label = u"Balance"
        for calls in self.get_history():
            if "Votre solde" in calls.label:
                detail.price = calls.price
                return detail
        detail.price = NotAvailable
        return detail



# -*- coding: utf-8 -*-

# Copyright(C) 2016      François Revol
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module
from weboob.capabilities.job import CapJob

from .browser import LinuxJobsBrowser


__all__ = ['LinuxJobsModule']


class LinuxJobsModule(Module, CapJob):
    NAME = 'linuxjobs'
    DESCRIPTION = u'linuxjobs website'
    MAINTAINER = u'François Revol'
    EMAIL = 'revol@free.fr'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    BROWSER = LinuxJobsBrowser

    def advanced_search_job(self):
        """
         Iter results of an advanced search

        :rtype: iter[:class:`BaseJobAdvert`]
        """
        raise NotImplementedError()

    def get_job_advert(self, _id, advert=None):
        """
        Get an announce from an ID.

        :param _id: id of the advert
        :type _id: str
        :param advert: the advert
        :type advert: BaseJobAdvert
        :rtype: :class:`BaseJobAdvert` or None if not found.
        """
        return self.browser.get_job_advert(_id, advert)

    def search_job(self, pattern=None):
        """
        Iter results of a search on a pattern.

        :param pattern: pattern to search on
        :type pattern: str
        :rtype: iter[:class:`BaseJobAdvert`]
        """
        for job_advert in self.browser.search_job(pattern):
            yield job_advert



# -*- coding: utf-8 -*-

# Copyright(C) 2017      Phyks (Lucas Verney)
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.backend import Module
from weboob.capabilities.base import find_object
from weboob.capabilities.gauge import CapGauge, GaugeSensor, SensorNotFound, Gauge

from .browser import RATPBrowser


__all__ = ['RATPModule']


class RATPSensor(GaugeSensor):
    def __init__(self, gauge):
        super(RATPSensor, self).__init__(id="%s_sensor" % gauge.id)
        self.name = "%s status" % (gauge.name)


class RATPModule(Module, CapGauge):
    NAME = 'ratp'
    DESCRIPTION = u'RATP network status'
    MAINTAINER = u'Phyks (Lucas Verney)'
    EMAIL = 'phyks@phyks.me'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    BROWSER = RATPBrowser

    def get_last_measure(self, sensor):
        """
        Get last measures of a sensor.

        :param sensor: ID of the sensor.
        :type sensor: str
        :rtype: :class:`GaugeMeasure`
        """
        # Note: The lower the value, the higher the perturbations.
        try:
            return next(self.browser.get_status(sensor))
        except KeyError:
            return SensorNotFound()

    def iter_gauges(self, pattern=None):
        """
        Iter gauges.

        :param pattern: if specified, used to search gauges.
        :type pattern: str
        :rtype: iter[:class:`Gauge`]
        """
        if not pattern:
            return self.browser.list_gauges()
        else:
            return [
                gauge
                for gauge in self.browser.list_gauges()
                if pattern in gauge.name
            ]

    def iter_sensors(self, gauge, pattern=None):
        """
        Iter instrument of a gauge.

        :param: ID of the gauge
        :param pattern: if specified, used to search sensors.
        :type pattern: str
        :rtype: iter[:class:`GaugeSensor`]
        """
        if not isinstance(gauge, Gauge):
            gauge = find_object(self.iter_gauges(), id=gauge, error=SensorNotFound)
        sensor = RATPSensor(gauge)
        sensor.lastvalue = self.get_last_measure(sensor.id)
        return [sensor]



# -*- coding: utf-8 -*-

# Copyright(C) 2017      Phyks (Lucas Verney)
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from weboob.browser.pages import HTMLPage
from weboob.browser.filters.standard import CleanDecimal, CleanText, Date, Env, Format
from weboob.browser.filters.html import Attr, Link, XPathNotFound
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.capabilities.base import NotAvailable
from weboob.capabilities.bill import Bill, Subscription
from weboob.tools.compat import urljoin


class LoginPage(HTMLPage):
    def do_login(self, username, password):
        form = self.get_form('//form[@class="Form"]')

        form["_username"] = username
        form["_password"] = password

        form.submit()


class MonBienPage(HTMLPage):
    @method
    class get_subscriptions(ListElement):
        item_xpath = '//li[has-class("MyPropertiesSelector-item") and not(has-class("MyPropertiesSelector-item--add"))]'

        class item(ItemElement):
            klass = Subscription

            def obj_id(self):
                link = Link('./a')(self)
                return link[link.rfind('/') + 1:]

            obj_label = Format(
                '%s - %s (%s)',
                CleanText('.//strong[has-class("MyPropertiesSelector-item-title")]'),
                CleanText('.//span[has-class("MyPropertiesSelector-item-desc")][1]'),
                CleanText('.//span[has-class("MyPropertiesSelector-item-desc")][last()]')
            )

            def obj_subscriber(self):
                subscriber = CleanText('//a[has-class("MainNav-item-logged")]')(self)
                subscriber = subscriber.replace('Bonjour', '').strip()
                return subscriber


class MesChargesPage(HTMLPage):
    @method
    class get_documents(ListElement):
        item_xpath = '//article[@data-taffy="utility_record"]'

        class item(ItemElement):
            klass = Bill

            obj_id = Format(
                '%s#%s',
                Env('subscription'),
                Attr('.', 'id')
            )

            obj_price = CleanDecimal('.//span[has-class("nbPrice")]',
                                     replace_dots=(',', '€'))

            obj_currency = "€"

            def obj_income(self):
                price = CleanText('.//span[has-class("nbPrice")]')(self)
                return not price.startswith('−')

            obj_label = CleanText('.//p[has-class("TeaserRow-desc")]')
            obj_date = Date(CleanText('.//p[has-class("TeaserRow-date")]'),
                            dayfirst=True)
            obj_duedate = obj_date
            obj_format = "pdf"

            def obj_url(self):
                try:
                    return urljoin(
                        self.page.browser.BASEURL,
                        Link('.//a[has-class("Download")]')(self)
                    )
                except XPathNotFound:
                    return NotAvailable



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Mathieu Jourdan
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from datetime import date

from weboob.deprecated.browser import Page
from weboob.capabilities.bill import Subscription


class LoginPage(Page):

    def login(self, login, password):
        self.browser.select_form('symConnexionForm')
        self.browser["portlet_login_plein_page_3{pageFlow.mForm.login}"] = unicode(login)
        self.browser["portlet_login_plein_page_3{pageFlow.mForm.password}"] = unicode(password)
        self.browser.submit()


class HomePage(Page):

    def on_loaded(self):
        pass


class AccountPage(Page):

    def get_subscription_list(self):
        table = self.document.xpath('//table[@id="ensemble_contrat_N0"]')[0]
        if len(table) > 0:
            # some clients may have subscriptions to gas and electricity,
            # but they receive a single bill
            # to avoid "boobill details" and "boobill bills" returning the same
            # table twice, we could return only one subscription for both.
            # We do not, and "boobill details" will take care of parsing only the
            # relevant section in the bill files.
            for line in table[0].xpath('//tbody/tr'):
                cells = line.xpath('td')
                snumber = cells[2].attrib['id'].replace('Contrat_', '')
                slabel = cells[0].xpath('a')[0].text.replace('offre', '').strip()
                d = unicode(cells[3].xpath('strong')[0].text.strip())
                sdate = date(*reversed([int(x) for x in d.split("/")]))
                sub = Subscription(snumber)
                sub._id = snumber
                sub.label = slabel
                sub.subscriber = unicode(cells[1])
                sub.renewdate = sdate
                yield sub


class TimeoutPage(Page):

    def on_loaded(self):
        pass



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2012 Romain Bignon, Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser.pages import HTMLPage
from weboob.browser.elements import ListElement, ItemElement, method
from weboob.browser.filters.standard import CleanText, Regexp, Field, Filter, debug
from weboob.capabilities.gauge import GaugeMeasure, GaugeSensor
from weboob.capabilities.base import NotAvailable


class Split(Filter):
    def __init__(self, selector, mode):
        super(Split, self).__init__(selector)
        self.mode = mode

    @debug()
    def filter(self, txt):
        if u"Temperatur" in txt:
            value = txt.split(': ')[1].split(u'°')[0]
            unit = u'°C'
        else:
            value = txt.split(':')[-1].split()[0]
            unit = txt.split(':')[-1].split()[1]
            if unit == u"W/m":
                unit = u"W/m²"
        try:
            value = float(value)
        except ValueError:
            value = NotAvailable
        return [value, unit][self.mode]


class StartPage(HTMLPage):

    @method
    class get_sensors_list(ListElement):
        item_xpath = '//p[@align="center"]'

        class item(ItemElement):
            klass = GaugeSensor

            obj_name = Regexp(CleanText('.'), '(.*?) {0,}: .*', "\\1")
            obj_id = CleanText(Regexp(Field('name'), '(.*)', "dd-\\1"), " .():")
            obj_gaugeid = u"wetter"
            obj_forecast = NotAvailable
            obj_unit = Split(CleanText('.'), 1)

            def obj_lastvalue(self):
                lastvalue = GaugeMeasure()
                lastvalue.level = Split(CleanText('.'), 0)(self)
                lastvalue.alarm = NotAvailable
                return lastvalue



# -*- coding: utf-8 -*-

# Copyright(C) 2012-2013  Fourcot Florent
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.test import BackendTest


class LeclercMobileTest(BackendTest):
    MODULE = 'leclercmobile'

    def test_list(self):
        """
        Test listing of subscriptions .
        No support of multi-account on the website, we could assume to
        have only one subscription.
        Check the balance if the subscription is ok.
        """
        subscriptions = list(self.backend.iter_subscription())
        self.assertTrue(len(subscriptions) == 1, msg="Account listing failed")
        self.assertTrue(self.backend.get_balance(subscriptions[0]) > 0,
                        msg="Get balance failed")

    def test_downloadbills(self):
        """
        Iter all bills and try to download it.
        """
        for subscription in self.backend.iter_subscription():
            for bill in self.backend.iter_documents(subscription.id):
                self.backend.download_document(bill.id)

    def test_history(self):
        for subscription in self.backend.iter_subscription():
            self.assertTrue(len(list(self.backend.iter_documents_history(subscription))) > 0)

    def test_details(self):
        for subscription in self.backend.iter_subscription():
            details = list(self.backend.get_details(subscription))
            self.assertTrue(len(details) > 5, msg="Not enough details")
            total = 0
            for d in details:
                total += d.price
            self.assertTrue(total > 0, msg="Parsing of price failed")



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re
from collections import OrderedDict

from weboob.capabilities.video import CapVideo, BaseVideo
from weboob.capabilities.collection import CapCollection, CollectionNotFound, Collection
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value
from weboob.exceptions import BrowserHTTPSDowngrade

from .browser import ArteBrowser
from .video import ArteVideo, ArteSiteVideo, VERSION_VIDEO, FORMATS, LANG, QUALITY, SITE


__all__ = ['ArteModule']


class ArteModule(Module, CapVideo, CapCollection):
    NAME = 'arte'
    MAINTAINER = u'Bezleputh'
    EMAIL = 'carton_ben@yahoo.fr'
    VERSION = '1.3'
    DESCRIPTION = 'Arte French and German TV'
    LICENSE = 'AGPLv3+'

    order = {'AIRDATE_DESC': 'Date',
             'VIEWS': 'Views',
             'ALPHA': 'Alphabetic',
             'LAST_CHANCE': 'Last chance'
             }

    versions_choice = OrderedDict([(k, u'%s' % (v.get('label'))) for k, v in VERSION_VIDEO.items])
    format_choice = OrderedDict([(k, u'%s' % (v)) for k, v in FORMATS.items])
    lang_choice = OrderedDict([(k, u'%s' % (v.get('label'))) for k, v in LANG.items])
    quality_choice = [u'%s' % (k) for k, v in QUALITY.items]

    CONFIG = BackendConfig(Value('lang', label='Lang of videos', choices=lang_choice, default='FRENCH'),
                           Value('order', label='Sort order', choices=order, default='AIRDATE_DESC'),
                           Value('quality', label='Quality of videos', choices=quality_choice, default='HD'),
                           Value('format', label='Format of videos', choices=format_choice, default=FORMATS.HTTP_MP4),
                           Value('version', label='Version of videos', choices=versions_choice))

    BROWSER = ArteBrowser

    def create_default_browser(self):
        return self.create_browser(lang=self.config['lang'].get(),
                                   quality=self.config['quality'].get(),
                                   order=self.config['order'].get(),
                                   format=self.config['format'].get(),
                                   version=self.config['version'].get())

    def parse_id(self, _id):
        sites = '|'.join(k.get('id') for k in SITE.values)
        m = re.match('^(%s)\.(.*)' % sites, _id)
        if m:
            return m.groups()

        m = re.match('https?://www.arte.tv/guide/\w+/(?P<id>.+)/(.*)', _id)
        if m:
            return SITE.PROGRAM.get('id'), m.group(1)

        m = re.match('https?://(%s).arte.tv/(\w+)/(.*)' % (sites), _id)
        if m:
            return m.group(1), '/%s/%s' % (m.group(2), m.group(3))

        if not _id.startswith('http'):
            return 'videos', _id

        return None, None

    def get_video(self, _id):
        site, _id = self.parse_id(_id)

        if not (site and _id):
            return None

        if site in [value.get('id') for value in SITE.values]:
            _site = (value for value in SITE.values if value.get('id') == site).next()
            return getattr(self.browser, _site.get('video'))(_id)

        else:
            return self.browser.get_video(_id)

    def search_videos(self, pattern, sortby=CapVideo.SEARCH_RELEVANCE, nsfw=False):
        return self.browser.search_videos(pattern)

    def fill_arte_video(self, video, fields):
        if fields != ['thumbnail']:
            video = self.browser.get_video(video.id, video)

        if 'thumbnail' in fields and video and video.thumbnail:
            video.thumbnail.data = self.browser.open(video.thumbnail.url).content

        if 'url' in fields and not video.url:
            video.ext, video.url = self.browser.fetch_url(video.id)

        return video

    def fill_site_video(self, video, fields):
        if fields != ['thumbnail']:
            for site in SITE.values:
                m = re.match('%s\.(.*)' % site.get('id'), video.id)
                if m:
                    video = getattr(self.browser, site.get('video'))(m.group(1), video)
                    break

        if 'thumbnail' in fields and video and video.thumbnail:
            try:
                video.thumbnail.data = self.browser.open(video.thumbnail.url).content
            except BrowserHTTPSDowngrade:
                pass

        return video

    def iter_resources(self, objs, split_path):
        if BaseVideo in objs:
            collection = self.get_collection(objs, split_path)
            if collection.path_level == 0:
                yield Collection([u'arte-latest'], u'Latest Arte videos')
                for site in SITE.values:
                    yield Collection([site.get('id')], site.get('label'))
            if collection.path_level == 1:
                if collection.split_path == [u'arte-latest']:
                    for video in self.browser.latest_videos():
                        yield video
                else:
                    for site in SITE.values:
                        if collection.split_path[0] == site.get('id') and collection.path_level in site.keys():
                            for item in getattr(self.browser, site.get(collection.path_level))():
                                yield item

            if collection.path_level >= 2:
                for site in SITE.values:
                    if collection.split_path[0] == site.get('id') and collection.path_level in site.keys():
                        for item in getattr(self.browser, site.get(collection.path_level))(collection.split_path):
                            yield item

    def validate_collection(self, objs, collection):
        if collection.path_level == 0:
            return
        if BaseVideo in objs and (collection.split_path == [u'arte-latest'] or
                                  collection.split_path[0] in [value.get('id') for value in SITE.values]):
            return
        if BaseVideo in objs and collection.path_level >= 2 and\
                collection.split_path[0] in [value.get('id') for value in SITE.values]:
            return
        raise CollectionNotFound(collection.split_path)

    OBJECTS = {ArteVideo: fill_arte_video, ArteSiteVideo: fill_site_video}



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.recipe import Recipe
from weboob.capabilities.base import NotAvailable
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.pages import HTMLPage
from weboob.browser.filters.standard import CleanText, Env, Regexp, Type, Join
from weboob.browser.filters.html import XPath


class ResultsPage(HTMLPage):
    """ Page which contains results as a list of recipies
    """
    @method
    class iter_recipes(ListElement):
        item_xpath = '//div[@id="searchlist"]/ul/li'

        class item(ItemElement):
            klass = Recipe

            def condition(self):
                return Regexp(CleanText('./h3/a/@href'), 'https://www.supertoinette.com/(recette)/\d*/.*',
                              default=None)(self)

            obj_id = Regexp(CleanText('./h3/a/@href'), 'https://www.supertoinette.com/recette/(.*).html', default=None)

            obj_title = CleanText('./h3/a')
            obj_short_description = CleanText('./p')


class RecipePage(HTMLPage):
    """ Page which contains a recipe
    """

    @method
    class get_recipe(ItemElement):
        klass = Recipe

        obj_id = Env('_id')
        obj_title = CleanText('//h1')
        obj_preparation_time = Type(Regexp(CleanText('//li[@class="time"]/span'), ".* (\d*) min"), type=int)

        obj_cooking_time = Type(Regexp(CleanText('//li[@class="time-cooking"]/span'), ".* (\d*) min"), type=int)

        def obj_nb_person(self):
            nb_pers = Regexp(CleanText('//div[@class="row ingredients"]/div/p'),
                             '.*pour (\d+) personnes', default=0)(self)
            return [nb_pers] if nb_pers else NotAvailable

        def obj_ingredients(self):
            i = []
            ingredients = XPath('//ul[@class="ingredientsList"]/li',
                                default=[])(self)
            for ingredient in ingredients:
                i.append(CleanText('.')(ingredient))
            return i

        obj_instructions = Join(u'\n- ', '//div[@class="recipe-prepa"]/ol/li', newline=True, addBefore='- ')

        obj_thumbnail_url = CleanText('//div[has-class("toprecipeImage")]/img/@src', default=NotAvailable)
        obj_picture_url = CleanText('//div[has-class("toprecipeImage")]/img/@src', default=NotAvailable)



# -*- coding: utf-8 -*-

# Copyright(C) 2011  Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.pages import HTMLPage, pagination
from weboob.browser.filters.standard import CleanText, Regexp, Env, Duration, DateTime
from weboob.browser.filters.html import Link

from weboob.capabilities.base import NotAvailable
from weboob.capabilities.video import BaseVideo
from weboob.capabilities.image import Thumbnail

from weboob.exceptions import ParseError
from weboob.tools.json import json

from datetime import timedelta
import re


def determine_ext(url, default_ext='unknown_video'):
    if url is None:
        return default_ext
    guess = url.partition('?')[0].rpartition('.')[2]
    if re.match(r'^[A-Za-z0-9]+$', guess):
        return guess
    elif guess.rstrip('/') in ('mp4', 'm3u8'):
        return guess.rstrip('/')
    else:
        return default_ext


class IndexPage(HTMLPage):
    @pagination
    @method
    class iter_videos(ListElement):
        item_xpath = '//div[@data-video-id]'
        next_page = Link('//a[@title="suivant"]')

        class item(ItemElement):
            klass = BaseVideo

            def validate(self, obj):
                return obj.id

            obj_id = CleanText('./div/@data-playable')
            obj_title = CleanText('./div[@class="media-block"]/h3')
            obj_author = CleanText('./div[@class="media-block"]/div/span/a')
            obj_duration = Duration(CleanText('./div/a/div[has-class("badge--duration")]'), default=NotAvailable)

            def obj_thumbnail(self):
                url = CleanText('./div/a/img/@data-src')(self)
                thumbnail = Thumbnail(url)
                thumbnail.url = url
                return thumbnail


class VideoPage(HTMLPage):

    @method
    class get_video(ItemElement):
        klass = BaseVideo

        obj_id = Env('_id')
        obj_title = CleanText('//title')
        obj_author = CleanText('//meta[@name="author"]/@content')
        obj_description = CleanText('//meta[@name="description"]/@content')

        def obj_duration(self):
            seconds = int(CleanText('//meta[@property="video:duration"]/@content', default=0)(self))
            return timedelta(seconds=seconds)

        def obj_thumbnail(self):
            url = CleanText('//meta[@property="og:image"]/@content')(self)
            thumbnail = Thumbnail(url)
            thumbnail.url = url
            return thumbnail

        obj_date = DateTime(CleanText('//meta[@property="video:release_date"]/@content'))

        def obj__formats(self):
            player = Regexp(CleanText('//script'), '.*var config = ({"context".*}}});\s*buildPlayer\(config\);.*', default=None)(self)
            if player:
                info = json.loads(player)
                if info.get('error') is not None:
                    raise ParseError(info['error']['title'])
                metadata = info.get('metadata')

                formats = {}
                for quality, media_list in metadata['qualities'].items():
                    for media in media_list:
                        media_url = media.get('url')
                        if not media_url:
                            continue
                        type_ = media.get('type')
                        if type_ == 'application/vnd.lumberjack.manifest':
                            continue
                        ext = determine_ext(media_url)
                        if ext in formats:
                            if quality in formats.get(ext):
                                formats[ext][quality] = media_url
                            else:
                                formats[ext] = {quality: media_url}
                        else:
                            formats[ext] = {quality: media_url}

                return formats
            return None



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Florent Fourcot
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.parcel import CapParcel, ParcelNotFound, Parcel
from weboob.tools.backend import Module

from .browser import ColissimoBrowser

__all__ = ['ColissimoModule']


class ColissimoModule(Module, CapParcel):
    NAME = 'colissimo'
    DESCRIPTION = u'Colissimo parcel tracking website'
    MAINTAINER = u'Florent Fourcot'
    EMAIL = 'weboob@flo.fourcot.fr'
    VERSION = '1.3'
    LICENSE = 'AGPLv3+'

    BROWSER = ColissimoBrowser

    def get_parcel_tracking(self, _id):
        # 13 is the magic length of colissimo tracking ids
        if len(_id) != 13:
            raise ParcelNotFound(u"Colissimo ID's must have 13 print character")

        events = self.browser.get_tracking_info(_id)
        p = Parcel(_id)
        p.history = events

        first = events[0]
        p.info = first.activity

        if u"remis au gardien ou" in p.info or u"Votre colis est livré" in p.info:
            p.status = p.STATUS_ARRIVED
        elif u"pas encore pris en charge par La Poste" in p.info:
            p.status = p.STATUS_PLANNED
        else:
            p.status = p.STATUS_IN_TRANSIT

        return p



# -*- coding: utf-8 -*-

# Copyright(C) 2017      Vincent A
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from collections import OrderedDict
from datetime import date

from weboob.browser import PagesBrowser, URL
from weboob.capabilities.messages import Message

from .pages import DatePage, IndexPage, ArticlePage


class BlogspotBrowser(PagesBrowser):
    BASEURL = 'http://www.blogspot.com'

    index = URL(r'/$', IndexPage)
    date = URL(r'/\?action=getTitles&widgetId=BlogArchive1&widgetType=BlogArchive&responseType=js&path=(?P<query>.*)', r'/(?P<year>\d+)/(?P<month>\d+)/$', DatePage)
    article = URL(r'/(?P<year>\d+)/(?P<month>\d+)/(?P<title>.*).html$', ArticlePage)

    def __init__(self, baseurl, *args, **kwargs):
        super(BlogspotBrowser, self).__init__(*args, **kwargs)
        self.BASEURL = baseurl
        self.cache = OrderedDict()

    def iter_dates(self):
        if not self.cache:
            self.index.go()
            for url in self.page.get_dates():
                m = self.date.match(url)
                key = (m.group('year'), m.group('month'))
                self.cache[key] = None

        for k in self.cache:
            yield self.build_date(k)

    def iter_articles(self, key):
        if self.cache[key] is None:
            query = self.date.build(year=key[0], month=key[1])
            self.date.go(query=query)
            self.cache[key] = list(self.page.get_articles())

        for j in self.cache[key]:
            yield self.build_article(j)

    def build_date(self, k):
        ret = Message(id='%s.%s' % k)
        ret.title = '%s/%s' % k
        ret.content = ''
        ret.date = date(int(k[0]), int(k[1]), 1)
        ret._type = 'date'
        ret._key = k
        return ret

    def build_article(self, j):
        m = self.article.match(j['url'])
        ret = Message(id=m.group('title'))
        ret.title = j['title']
        ret.url = j['url']
        ret.flags = Message.IS_HTML
        ret.date = date(int(m.group('year')), int(m.group('month')), 1)
        ret.children = []
        ret._type = 'article'
        return ret

    def get_article(self, url):
        self.location(url)
        assert self.article.is_here()
        return self.page.get_message()



# -*- coding: utf-8 -*-

# Copyright(C) 2012 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.housing import Query, TypeNotSupported
from weboob.tools.compat import urlencode

from weboob.browser import PagesBrowser, URL
from .pages import SearchResultsPage, HousingPage, CitiesPage
from weboob.browser.profiles import Android

__all__ = ['SeLogerBrowser']


class SeLogerBrowser(PagesBrowser):
    BASEURL = 'http://www.seloger.com'
    PROFILE = Android()
    cities = URL('js,ajax,villequery_v3.htm\?ville=(?P<pattern>.*)', CitiesPage)
    search = URL('http://ws.seloger.com/search.xml\?(?P<request>.*)', SearchResultsPage)
    housing = URL('http://ws.seloger.com/annonceDetail.xml\?idAnnonce=(?P<_id>\d+)&noAudiotel=(?P<noAudiotel>\d)',
                  HousingPage)

    def search_geo(self, pattern):
        return self.cities.open(pattern=pattern).iter_cities()

    TYPES = {Query.TYPE_RENT: 1,
             Query.TYPE_SALE: 2,
             }

    RET = {Query.HOUSE_TYPES.HOUSE: '2',
           Query.HOUSE_TYPES.APART: '1',
           Query.HOUSE_TYPES.LAND: '4',
           Query.HOUSE_TYPES.PARKING: '3',
           Query.HOUSE_TYPES.OTHER: '10'}

    def search_housings(self, type, cities, nb_rooms, area_min, area_max, cost_min, cost_max, house_types):

        if type not in self.TYPES:
            raise TypeNotSupported()

        data = {'ci':            ','.join(cities),
                'idtt':          self.TYPES.get(type, 1),
                'org':           'advanced_search',
                'surfacemax':    area_max or '',
                'surfacemin':    area_min or '',
                'tri':           'd_dt_crea',
                }

        if type == Query.TYPE_SALE:
            data['pxmax'] = cost_max or ''
            data['pxmin'] = cost_min or ''
        else:
            data['px_loyermax'] = cost_max or ''
            data['px_loyermin'] = cost_min or ''

        if nb_rooms:
            data['nb_pieces'] = nb_rooms

        ret = []
        for house_type in house_types:
            if house_type in self.RET:
                ret.append(self.RET.get(house_type))

        if ret:
            data['idtypebien'] = ','.join(ret)

        return self.search.go(request=urlencode(data)).iter_housings()

    def get_housing(self, _id, obj=None):
        return self.housing.go(_id=_id, noAudiotel=1).get_housing(obj=obj)



# -*- coding: utf-8 -*-

# Copyright(C) 2015 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import time

from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword
from weboob.capabilities.bank import AccountNotFound
from weboob.capabilities.base import find_object


from .pages import LoginPage, MenuPage, AccountsPage, HistoryPage, IbanPage, ErrorPage


__all__ = ['DelubacBrowser']


class DelubacBrowser(LoginBrowser):
    BASEURL = 'https://e.delubac.com'

    home = URL('/es@b/fr/esab.jsp')
    login = URL('/es@b/fr/codeident.jsp',
                '/es@b/servlet/internet0.ressourceWeb.servlet.Login', LoginPage)
    menu = URL('/es@b/fr/menuConnecte1.jsp\?c&deploye=false&pulseMenu=false&styleLien=false&dummyDate=(?P<date>.*)', MenuPage)
    accounts = URL('/es@b/servlet/internet0.ressourceWeb.servlet.EsabServlet.*', AccountsPage)
    history = URL('/es@b/servlet/internet0.ressourceWeb.servlet.ListeDesMouvementsServlet.*', HistoryPage)
    error = URL('/es@b/servlet/internet0.ressourceWeb.servlet.*', ErrorPage)
    iban = URL('/es@b/fr/rib.jsp', IbanPage)

    def do_login(self):
        self.home.go()
        self.login.go()

        self.page.login(self.username, self.password)

        if self.page.incorrect_auth:
            raise BrowserIncorrectPassword()

    @need_login
    def iter_accounts(self):
        self.menu.go(date=int(time.time()*1000))
        self.location(self.page.accounts_url)
        for account in self.page.get_list():
            self.location(account._rib_link)
            account.iban = self.page.get_iban()
            yield account

    @need_login
    def get_account(self, _id):
        return find_object(self.iter_accounts(), id=_id, error=AccountNotFound)

    @need_login
    def iter_history(self, account):
        self.location(account._link)
        if self.error.is_here():
            return iter([])
        return self.page.get_transactions()



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.
import requests

from .job import APEC_CONTRATS, APEC_EXPERIENCE

from weboob.browser.elements import ItemElement, method, DictElement
from weboob.browser.pages import JsonPage, pagination
from weboob.browser.filters.standard import DateTime, Format, Regexp
from weboob.browser.filters.json import Dict
from weboob.browser.filters.html import CleanHTML
from weboob.capabilities.job import BaseJobAdvert
from weboob.capabilities.base import NotAvailable


class IdsPage(JsonPage):

    def get_adverts_number(self):
        return self.doc['totalCount']

    @pagination
    @method
    class iter_job_adverts(DictElement):
        item_xpath = 'resultats'

        def next_page(self):
            self.page.browser.start += self.env['range']
            if self.page.browser.start <= self.env['count']:
                data = self.page.browser.create_parameters(pattern=self.env['pattern'],
                                                           fonctions=self.env['fonctions'],
                                                           lieux=self.env['lieux'],
                                                           secteursActivite=self.env['secteursActivite'],
                                                           typesContrat=self.env['typesContrat'],
                                                           typesConvention=self.env['typesConvention'],
                                                           niveauxExperience=self.env['niveauxExperience'],
                                                           salaire_min=self.env['salaire_min'],
                                                           salaire_max=self.env['salaire_max'],
                                                           date_publication=self.env['date_publication'],
                                                           start=self.page.browser.start,
                                                           range=self.env['range'])

                return requests.Request("POST", self.page.url, data=data)

        class item(ItemElement):
            klass = BaseJobAdvert
            obj_id = Regexp(Dict('@uriOffre'), '.*=(.*)')


class OffrePage(JsonPage):
    @method
    class get_job_advert(ItemElement):
        klass = BaseJobAdvert

        obj_id = Dict('numeroOffre')
        obj_title = Dict('intitule')
        obj_description = CleanHTML(Dict('texteHtml'))
        obj_job_name = Dict('intitule')
        obj_publication_date = DateTime(Dict('datePublication'))
        obj_society_name = Dict('nomCommercialEtablissement', default=NotAvailable)

        def obj_contract_type(self):
            ctr = '%s' % Dict('idNomTypeContrat')(self)
            return APEC_CONTRATS.get(ctr) if ctr in APEC_CONTRATS else NotAvailable

        obj_place = Dict('lieux/0/libelleLieu')
        obj_pay = Dict('salaireTexte')

        def obj_experience(self):
            exp = u'%s' % Dict('idNomNiveauExperience')(self)
            return APEC_EXPERIENCE.get(exp) if exp in APEC_EXPERIENCE else NotAvailable

        obj_url = Format('https://cadres.apec.fr/home/mes-offres/recherche-des-offres-demploi/liste-des-offres-demploi/detail-de-loffre-demploi.html?numIdOffre=%s', Dict('numeroOffre'))



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Roger Philibert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.capabilities.video import CapVideo, BaseVideo
from weboob.capabilities.collection import CapCollection, CollectionNotFound
from weboob.tools.backend import Module

from .browser import JacquieEtMichelBrowser
from .video import JacquieEtMichelVideo


__all__ = ['JacquieEtMichelModule']


class JacquieEtMichelModule(Module, CapVideo, CapCollection):
    NAME = 'jacquieetmichel'
    MAINTAINER = u'Roger Philibert'
    EMAIL = 'roger.philibert@gmail.com'
    VERSION = '1.3'
    DESCRIPTION = 'Jacquie et Michel TV'
    LICENSE = 'AGPLv3+'
    BROWSER = JacquieEtMichelBrowser

    def get_video(self, _id):
        with self.browser:
            video = self.browser.get_video(_id)
        return video

    def search_videos(self, pattern, sortby=CapVideo.SEARCH_RELEVANCE, nsfw=False):
        if not nsfw:
            return iter([])

        with self.browser:
            return self.browser.search_videos(pattern)

    def fill_video(self, video, fields):
        if fields != ['thumbnail']:
            # if we don't want only the thumbnail, we probably want also every fields
            with self.browser:
                video = self.browser.get_video(JacquieEtMichelVideo.id2url(video.id), video)
        if 'thumbnail' in fields and video.thumbnail:
            with self.browser:
                video.thumbnail.data = self.browser.readurl(video.thumbnail.url)

        return video

    def iter_resources(self, objs, split_path):
        if BaseVideo in objs:
            collection = self.get_collection(objs, split_path)
            if collection.path_level == 0:
                yield self.get_collection(objs, [u'latest_nsfw'])
            if collection.split_path == [u'latest_nsfw']:
                for video in self.browser.latest_videos():
                    yield video

    def validate_collection(self, objs, collection):
        if collection.path_level == 0:
            return
        if BaseVideo in objs and collection.split_path == [u'latest_nsfw']:
            collection.title = u'Latest Jacquie & Michel videos (NSFW)'
            return
        raise CollectionNotFound(collection.split_path)

    OBJECTS = {JacquieEtMichelVideo: fill_video}



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Roger Philibert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.deprecated.browser import Page
from weboob.capabilities.image import Thumbnail

from datetime import datetime
import re

from .gallery import EHentaiGallery


class LoginPage(Page):
    def is_logged(self):
        success_p = self.document.xpath(
                '//p[text() = "Login Successful. You will be returned momentarily."]')
        if len(success_p):
            return True
        else:
            return False


class HomePage(Page):
    pass


class IndexPage(Page):
    def iter_galleries(self):
        lines = self.document.xpath('//table[@class="itg"]//tr[@class="gtr0" or @class="gtr1"]')
        for line in lines:
            a = line.xpath('.//div[@class="it3"]/a')[-1]
            url = a.attrib["href"]
            title = a.text.strip()
            yield EHentaiGallery(re.search('(?<=/g/)\d+/[\dabcdef]+', url).group(0), title=title)


class GalleryPage(Page):
    def image_pages(self):
        return self.document.xpath('//div[@class="gdtm"]//a/attribute::href')

    def _page_numbers(self):
        return [n for n in self.document.xpath("(//table[@class='ptt'])[1]//td/text()") if re.match(r"\d+", n)]

    def gallery_exists(self, gallery):
        if self.document.xpath("//h1"):
            return True
        else:
            return False

    def fill_gallery(self, gallery):
        gallery.title = self.document.xpath("//h1[@id='gn']/text()")[0]
        cardinality_string = self.document.xpath("//div[@id='gdd']//tr[td[@class='gdt1']/text()='Length:']/td[@class='gdt2']/text()")[0]
        gallery.cardinality = int(re.match(r"\d+", cardinality_string).group(0))
        date_string = self.document.xpath("//div[@id='gdd']//tr[td[@class='gdt1']/text()='Posted:']/td[@class='gdt2']/text()")[0]
        gallery.date = datetime.strptime(date_string, "%Y-%m-%d %H:%M")
        rating_string = self.document.xpath("//td[@id='rating_label']/text()")[0]
        rating_match = re.search(r"\d+\.\d+", rating_string)
        if rating_match is None:
            gallery.rating = None
        else:
            gallery.rating = float(rating_match.group(0))

        gallery.rating_max = 5

        try:
            thumbnail_url = self.document.xpath("//div[@class='gdtm']/a/img/attribute::src")[0]
        except IndexError:
            thumbnail_style = self.document.xpath("//div[@class='gdtm']/div/attribute::style")[0]
            thumbnail_url = re.search(r"background:[^;]+url\((.+?)\)", thumbnail_style).group(1)

        gallery.thumbnail = Thumbnail(thumbnail_url)


class ImagePage(Page):
    def get_url(self):
        return self.document.xpath('//div[@class="sni"]/a/img/attribute::src')[0]



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Oleg Plakhotniuk
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from requests.exceptions import Timeout, ConnectionError

from weboob.capabilities.bank import AccountNotFound, Account, Transaction
from weboob.tools.capabilities.bank.transactions import \
    AmericanTransaction as AmTr
from weboob.browser import LoginBrowser, URL, need_login
from weboob.browser.exceptions import ServerError
from weboob.browser.pages import HTMLPage
from weboob.exceptions import BrowserIncorrectPassword
from datetime import datetime


__all__ = ['VicSecCard']


class SomePage(HTMLPage):
    @property
    def logged(self):
        return bool(self.doc.xpath(u'//span[text()="Sign Out"]'))


class LoginPage(SomePage):
    def login(self, username, password):
        form = self.get_form(name='frmLogin')
        form['username_input'] = username
        form['userName'] = username
        form['password_input'] = password
        form['hiddenPassword'] = password
        form['btnLogin'] = 'btnLogin'
        form.submit()


class HomePage(SomePage):
    def account(self):
        id_ = self.doc.xpath(u'//strong[contains(text(),'
            '"Credit Card account ending in")]/text()')[0].strip()[-4:]
        balance = self.doc.xpath(u'//span[@class="description" and text()="Current Balance"]/../span[@class="total"]/text()')[0].strip()
        cardlimit = self.doc.xpath(u'//span[contains(text(),"Credit limit")]'
                                    '/text()')[0].split()[-1]
        paymin = self.doc.xpath(u'//section[@id=" account_summary"]'
            '//strong[text()="Minimum Payment Due"]/../../span[2]/text()'
            )[0].strip()
        a = Account()
        a.id = id_
        a.label = u'ACCOUNT ENDING IN %s' % id_
        a.currency = Account.get_currency(balance)
        a.balance = -AmTr.decimal_amount(balance)
        a.type = Account.TYPE_CARD
        a.cardlimit = AmTr.decimal_amount(cardlimit)
        a.paymin = AmTr.decimal_amount(paymin)
        #TODO: Add paydate.
        #Oleg: I don't have an account with scheduled payment.
        #      Need to wait for a while...
        return a


class RecentPage(SomePage):
    def iter_transactions(self):
        for li in self.doc.xpath('//section[@class="transactions"]//div/li'):
            date = li.xpath('p[@data-type="date"]//text()')[0].strip()
            label = li.xpath('p[@data-type="description"]//text()')[0].strip()
            amount = li.xpath('p[@data-type="amount"]//text()')[0].strip()
            t = Transaction()
            t.date = datetime.strptime(date, '%m/%d/%Y')
            t.rdate = datetime.strptime(date, '%m/%d/%Y')
            t.type = Transaction.TYPE_UNKNOWN
            t.raw = unicode(label)
            t.label = unicode(label)
            t.amount = -AmTr.decimal_amount(amount)
            yield t


class VicSecCard(LoginBrowser):
    BASEURL = 'https://c.comenity.net'
    MAX_RETRIES = 10
    TIMEOUT = 30.0
    login = URL(r'/victoriassecret/$', LoginPage)
    home = URL(r'/victoriassecret/secure/SecureHome.xhtml', HomePage)
    recent = URL(r'/victoriassecret/secure/accountactivity/Transactions.xhtml',
                 RecentPage)
    unknown = URL('.*', SomePage)

    def get_account(self, id_):
        a = next(self.iter_accounts())
        if (a.id != id_):
            raise AccountNotFound()
        return a

    @need_login
    def iter_accounts(self):
        yield self.home.stay_or_go().account()

    @need_login
    def iter_history(self, account):
        for trans in self.recent.stay_or_go().iter_transactions():
            yield trans

    def do_login(self):
        self.session.cookies.clear()
        self.login.go().login(self.username, self.password)
        if not self.page.logged:
            raise BrowserIncorrectPassword()

    def location(self, *args, **kwargs):
        for i in xrange(self.MAX_RETRIES):
            try:
                return super(VicSecCard, self).location(*args, **kwargs)
            except (ServerError, Timeout, ConnectionError) as e:
                pass
        raise e



# -*- coding: utf-8 -*-

# Copyright(C) 2012 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re

from weboob.browser import LoginBrowser, URL, need_login
from weboob.exceptions import BrowserIncorrectPassword
from weboob.capabilities.bank import Account

from .pages import LoginPage, AccountsPage, ProAccountsPage, TransactionsPage, \
                   ProTransactionsPage, IbanPage, RedirectPage, AVPage


class CreditDuNordBrowser(LoginBrowser):
    ENCODING = 'UTF-8'

    login = URL('$',
                '/.*\?.*_pageLabel=page_erreur_connexion',                        LoginPage)
    redirect = URL('/swm/redirectCDN.html',                                       RedirectPage)
    av = URL('/vos-comptes/particuliers/V1_transactional_portal_page_',           AVPage)
    accounts = URL('/vos-comptes/particuliers',
                   '/vos-comptes/particuliers/transac_tableau_de_bord',           AccountsPage)
    transactions = URL('/vos-comptes/.*/transac/particuliers',                    TransactionsPage)
    proaccounts = URL('/vos-comptes/(professionnels|entreprises)',                ProAccountsPage)
    protransactions = URL('/vos-comptes/.*/transac/(professionnels|entreprises)', ProTransactionsPage)
    loans = URL('/vos-comptes/professionnels/credit_en_cours',                    ProAccountsPage)
    iban = URL('/vos-comptes/IPT/cdnProxyResource/transacClippe/RIB_impress.asp', IbanPage)

    account_type = 'particuliers'

    def __init__(self, website, *args, **kwargs):
        super(CreditDuNordBrowser, self).__init__(*args, **kwargs)
        self.BASEURL = "https://%s" % website

    def is_logged(self):
        return self.page is not None and not self.login.is_here() and \
            not self.page.doc.xpath(u'//b[contains(text(), "vous devez modifier votre code confidentiel")]')

    def home(self):
        if self.is_logged():
            self.location('/vos-comptes/%s' % self.account_type)
            self.location(self.page.doc.xpath(u'//a[contains(text(), "Synthèse")]')[0].attrib['href'])
        else:
            self.do_login()

    def do_login(self):
        self.login.go().login(self.username, self.password)

        if self.login.is_here():
            raise BrowserIncorrectPassword(self.page.get_error())

        if not self.is_logged():
            raise BrowserIncorrectPassword()

        m = re.match('https://[^/]+/vos-comptes/(\w+).*', self.url)
        if m:
            self.account_type = m.group(1)

    @need_login
    def _iter_accounts(self):
        self.home()
        self.location(self.page.get_av_link())
        if self.av.is_here():
            for a in self.page.get_av_accounts():
                self.location(a._link, data=a._args)
                self.location(a._link.replace("_attente", "_detail_contrat_rep"), data=a._args)
                self.page.fill_diff_currency(a)
                yield a
        self.home()
        for a in self.page.get_list():
            yield a
        self.loans.go()
        for a in self.page.get_list():
            yield a

    @need_login
    def get_accounts_list(self):
        accounts = list(self._iter_accounts())

        self.page.iban_page()

        link = self.page.iban_go()

        if self.page.has_iban():
            for a in [a for a in accounts if a._acc_nb]:
                self.location(link + a._acc_nb)
                a.iban = self.page.get_iban()

        return accounts

    def get_account(self, id):
        for a in self._iter_accounts():
            if a.id == id:
                return a
        return None

    @need_login
    def iter_transactions(self, link, args, acc_type):
        if args is None:
            return

        while args is not None:
            self.location(link, data=args)

            assert self.transactions.is_here()

            for tr in self.page.get_history(acc_type):
                yield tr

            args = self.page.get_next_args(args)

    @need_login
    def get_history(self, account, coming=False):
        if coming and account.type is not Account.TYPE_CARD or account.type is Account.TYPE_LOAN:
            return []

        transactions = []
        for tr in self.iter_transactions(account._link, account._args, account.type):
            transactions.append(tr)
        return transactions

    @need_login
    def get_investment(self, account):
        if not account._inv:
            return []

        investments = []
        if account.type in (Account.TYPE_MARKET, Account.TYPE_PEA):
            self.location(account._link, data=account._args)
            investments = [i for i in self.page.get_market_investment()]
        elif (account.type == Account.TYPE_LIFE_INSURANCE):
            self.location(account._link, data=account._args)
            self.location(account._link.replace("_attente", "_detail_contrat_rep"), data=account._args)
            investments = [i for i in self.page.get_deposit_investment()]
        return investments

    @need_login
    def get_profile(self):
        self.home()
        return self.page.get_profile()



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re
from datetime import datetime, time, timedelta
from decimal import Decimal

from mechanize import TextControl

from weboob.capabilities.base import Currency, UserError
from weboob.deprecated.browser import Page
from weboob.tools.json import json


class ForeignPage(Page):
    def on_loaded(self):
        raise UserError('Your IP address is localized in a country not supported by this module (%s). Currently only the French website is supported.' % self.group_dict['country'])


class CitiesPage(Page):
    def get_stations(self):
        result = json.loads(self.document[self.document.find('{'):-2])
        return result['CITIES']


class SearchPage(Page):
    def search(self, departure, arrival, date, age, card, comfort_class):
        self.browser.select_form(name='saisie')
        self.browser['ORIGIN_CITY'] = departure.encode(self.browser.ENCODING)
        self.browser['DESTINATION_CITY'] = arrival.encode(self.browser.ENCODING)

        if date is None:
            date = datetime.now() + timedelta(hours=1)
        elif date < datetime.now():
            raise UserError("You cannot look for older departures")

        self.browser['OUTWARD_DATE'] = date.strftime('%d/%m/%y')
        self.browser['OUTWARD_TIME'] = [str(date.hour)]
        self.browser['PASSENGER_1'] = [age]
        self.browser['PASSENGER_1_CARD'] = [card]
        self.browser['COMFORT_CLASS'] = [str(comfort_class)]
        self.browser.controls.append(TextControl('text', 'nbAnimalsForTravel', {'value': ''}))
        self.browser['nbAnimalsForTravel'] = '0'
        self.browser.submit()


class SearchErrorPage(Page):
    def on_loaded(self):
        p = self.document.getroot().cssselect('div.messagesError p')
        if len(p) > 0:
            message = p[0].text.strip()
            raise UserError(message)


class SearchInProgressPage(Page):
    def on_loaded(self):
        link = self.document.xpath('//a[@id="url_redirect_proposals"]')[0]
        self.browser.location(link.attrib['href'])


class ResultsPage(Page):
    def get_value(self, div, name, last=False):
        i = -1 if last else 0
        p = div.cssselect(name)[i]
        sub = p.find('p')
        if sub is not None:
            txt = sub.tail.strip()
            if txt == '':
                p.remove(sub)
            else:
                return unicode(txt)

        return unicode(self.parser.tocleanstring(p))

    def parse_hour(self, div, name, last=False):
        txt = self.get_value(div, name, last)
        hour, minute = map(int, txt.split('h'))
        return time(hour, minute)

    def iter_results(self):
        for div in self.document.getroot().cssselect('div.train_info'):
            info = None
            price = None
            currency = None
            for td in div.cssselect('td.price'):
                txt = self.parser.tocleanstring(td)
                p = Decimal(re.sub('([^\d\.]+)', '', txt))
                if price is None or p < price:
                    info = list(div.cssselect('strong.price_label')[0].itertext())[-1].strip().strip(':')
                    price = p
                    currency = Currency.get_currency(txt)

            yield {'type': self.get_value(div, 'div.transporteur-txt'),
                   'time': self.parse_hour(div, 'div.departure div.hour'),
                   'departure': self.get_value(div, 'div.departure div.station'),
                   'arrival': self.get_value(div, 'div.arrival div.station', last=True),
                   'arrival_time': self.parse_hour(div, 'div.arrival div.hour', last=True),
                   'price': price,
                   'currency': currency,
                   'price_info': info,
                  }



# -*- coding: utf-8 -*-

# Copyright(C) 2017      Phyks (Lucas Verney)
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import datetime

from weboob.browser.pages import JsonPage, HTMLPage, pagination
from weboob.browser.filters.standard import CleanDecimal, CleanText, Date, Env, Format, Regexp
from weboob.browser.filters.html import Attr, Link
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.capabilities.base import NotAvailable
from weboob.capabilities.housing import City, Housing, UTILITIES
from weboob.tools.capabilities.housing.housing import PricePerMeterFilter
from weboob.tools.compat import urljoin

from .constants import AVAILABLE_TYPES, QUERY_TYPES, QUERY_HOUSE_TYPES


class CitiesPage(JsonPage):
    def iter_cities(self):
        cities_list = self.doc
        if isinstance(self.doc, dict):
            cities_list = self.doc.values()

        for city in cities_list:
            city_obj = City()
            city_obj.id = city
            city_obj.name = city
            yield city_obj


class HousingPage(HTMLPage):
    @method
    class get_housing(ItemElement):
        klass = Housing

        obj_id = Format(
            '%s:%s',
            Env('type'),
            Attr('//div[boolean(@data-property-reference)]', 'data-property-reference')
        )

        def obj_url(self):
            return self.page.url

        obj_title = CleanText('//h1[has-class("OfferTop-title")]')
        obj_area = CleanDecimal(
            Regexp(
                CleanText(
                    '//div[has-class("MiniData")]//p[has-class("MiniData-item")][1]'
                ),
                r'(\d*\.*\d*) .*'
            )
        )
        obj_cost = CleanDecimal(
            Regexp(
                CleanText(
                    '//p[has-class("OfferTop-price")]'
                ),
                r'([\d \.]*) .*'
            )
        )
        obj_price_per_meter = PricePerMeterFilter()
        obj_currency = Regexp(
            CleanText(
                '//p[has-class("OfferTop-price")]'
            ),
            r'[\d \.]* (.) .*'
        )
        obj_location = Format(
            '%s - %s',
            CleanText('//p[@data-behat="adresseBien"]'),
            CleanText('//p[has-class("OfferTop-loc")]')
        )
        obj_text = CleanText('//div[has-class("OfferDetails-content")]/p[1]')
        obj_phone = Regexp(
            Link(
                '//a[has-class("OfferContact-btn--tel")]'
            ),
            r'tel:(.*)'
        )

        def obj_photos(self):
            photos = []
            for photo in self.xpath('//li[has-class("OfferSlider-thumbs-item")]/img'):
                photos.append(Attr('.', 'src')(photo))
            return photos

        obj_date = datetime.date.today()

        def obj_utilities(self):
            price = CleanText(
                '//p[has-class("OfferTop-price")]'
            )(self)
            if "charges comprises" in price.lower():
                return UTILITIES.INCLUDED
            else:
                return UTILITIES.EXCLUDED

        obj_rooms = CleanDecimal(
            '//div[has-class("MiniData")]//p[has-class("MiniData-item")][2]'
        )
        obj_bedrooms = CleanDecimal(
            '//div[has-class("MiniData")]//p[has-class("MiniData-item")][3]',
            default=NotAvailable
        )

        def obj_details(self):
            details = {
                "dispo": Date(
                    Regexp(
                        CleanText('//p[has-class("OfferTop-dispo")]'),
                        r'.* (\d\d\/\d\d\/\d\d\d\d)',
                        default=datetime.date.today().isoformat()
                    )
                )(self),
                "priceMentions": CleanText('//p[has-class("OfferTop-mentions")]')(self),
                "agency": CleanText('//p[has-class("OfferContact-address")]')(self)
            }
            for item in self.xpath('//div[has-class("OfferDetails-columnize")]/div'):
                category = CleanText('./h3[has-class("OfferDetails-title--2")]')(item)
                details[category] = {}
                for detail_item in item.xpath('.//ul[has-class("List--data")]/li'):
                    detail_title = CleanText('.//span[has-class("List-data")]')(detail_item)
                    detail_value = CleanText('.//*[has-class("List-value")]')(detail_item)
                    details[category][detail_title] = detail_value
                for detail_item in item.xpath('.//ul[has-class("List--bullet")]/li'):
                    detail_title = CleanText('.')(detail_item)
                    details[category][detail_title] = True
            electric_consumption = CleanDecimal(Regexp(
                Attr('//div[has-class("OfferDetails-content")]//img', 'src'),
                r'https://dpe.foncia.net\/(\d+)\/.*',
                default=None
            )(self))
            if electric_consumption is not None:
                details["electric_consumption"] = '{} kWhEP/m².an'.format(electric_consumption)
                if electric_consumption <= 50:
                    details["DPE"] = "A"
                elif electric_consumption > 50 and electric_consumption <= 90:
                    details["DPE"] = "B"
                elif electric_consumption > 90 and electric_consumption <= 150:
                    details["DPE"] = "C"
                elif electric_consumption > 150 and electric_consumption <= 230:
                    details["DPE"] = "D"
                elif electric_consumption > 230 and electric_consumption <= 330:
                    details["DPE"] = "E"
                elif electric_consumption > 330 and electric_consumption <= 450:
                    details["DPE"] = "F"
                else:
                    details["DPE"] = "G"
            else:
                details["electric_consumption"] = NotAvailable
                details["DPE"] = NotAvailable
            return details


class SearchPage(HTMLPage):
    def do_search(self, query, cities):
        form = self.get_form('//form[@name="searchForm"]')

        form['searchForm[type]'] = QUERY_TYPES.get(query.type, None)
        form['searchForm[localisation]'] = cities
        form['searchForm[type_bien][]'] = []
        for house_type in query.house_types:
            try:
                form['searchForm[type_bien][]'].extend(
                    QUERY_HOUSE_TYPES[house_type]
                )
            except KeyError:
                pass
        form['searchForm[type_bien][]'] = [
            x for x in form['searchForm[type_bien][]']
            if x in AVAILABLE_TYPES.get(query.type, [])
        ]
        if query.area_min:
            form['searchForm[surface_min]'] = query.area_min
        if query.area_max:
            form['searchForm[surface_max]'] = query.area_max
        if query.cost_min:
            form['searchForm[prix_min]'] = query.cost_min
        if query.cost_max:
            form['searchForm[prix_max]'] = query.cost_max
        if query.nb_rooms:
            form['searchForm[pieces]'] = [i for i in range(1, query.nb_rooms + 1)]
        form.submit()

    def find_housing(self, query_type, housing):
        form = self.get_form('//form[@name="searchForm"]')
        form['searchForm[type]'] = query_type
        form['searchForm[reference]'] = housing
        form.submit()


class SearchResultsPage(HTMLPage):
    @pagination
    @method
    class iter_housings(ListElement):
        item_xpath = '//article[has-class("TeaserOffer")]'

        next_page = Link('//div[has-class("Pagination--more")]/a[contains(text(), "Suivant")]')

        class item(ItemElement):
            klass = Housing

            obj_id = Format(
                '%s:%s',
                Env('type'),
                Attr('.//span[boolean(@data-reference)]', 'data-reference')
            )

            def obj_url(self):
                return urljoin(
                    self.page.browser.BASEURL,
                    Link('.//h3[has-class("TeaserOffer-title")]/a')(self)
                )

            obj_title = CleanText('.//h3[has-class("TeaserOffer-title")]')
            obj_area = CleanDecimal(
                Regexp(
                    CleanText(
                        './/div[has-class("MiniData")]//p[@data-behat="surfaceDesBiens"]'
                    ),
                    r'(\d*\.*\d*) .*'
                )
            )
            obj_cost = CleanDecimal(
                Regexp(
                    CleanText(
                        './/strong[has-class("TeaserOffer-price-num")]'
                    ),
                    r'([\d \.]*) .*'
                )
            )
            obj_price_per_meter = PricePerMeterFilter()
            obj_currency = Regexp(
                CleanText(
                    './/strong[has-class("TeaserOffer-price-num")]'
                ),
                r'[\d \.]* (.) .*'
            )
            obj_location = CleanText('.//p[has-class("TeaserOffer-loc")]')
            obj_text = CleanText('.//p[has-class("TeaserOffer-description")]')

            def obj_photos(self):
                return [
                    Attr('.//a[has-class("TeaserOffer-ill")]/img', 'src')(self)
                ]

            obj_date = datetime.date.today()

            def obj_utilities(self):
                price = CleanText(
                    './/strong[has-class("TeaserOffer-price-num")]'
                )(self)
                if "charges comprises" in price.lower():
                    return UTILITIES.INCLUDED
                else:
                    return UTILITIES.EXCLUDED

            obj_rooms = CleanDecimal(
                './/div[has-class("MiniData")]//p[@data-behat="nbPiecesDesBiens"]'
            )
            obj_bedrooms = NotAvailable

            def obj_details(self):
                return {
                    "dispo": Date(
                        Attr('.//span[boolean(@data-dispo)]', 'data-dispo',
                             default=datetime.date.today().isoformat())
                    )(self),
                    "priceMentions": CleanText('.//span[has-class("TeaserOffer-price-mentions")]')(self)
                }



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from .calendar import RazibusCalendarEvent

from datetime import time

from weboob.browser.pages import HTMLPage
from weboob.browser.elements import ItemElement, ListElement, method
from weboob.browser.filters.html import CleanHTML, Link
from weboob.browser.filters.standard import Regexp, CleanText, DateTime, CombineDate, Filter, Env


class EndTime(Filter):
    def filter(self, el):
        return time.max


class EventListPage(HTMLPage):
    @method
    class list_events(ListElement):
        item_xpath = '//div[@class="item"]'

        class item(ItemElement):
            klass = RazibusCalendarEvent

            def validate(self, obj):
                return (self.is_valid_event(obj, self.env['city'], self.env['categories']) and
                        self.is_event_in_valid_period(obj.start_date, self.env['date_from'], self.env['date_to']))

            def is_valid_event(self, event, city, categories):
                if city and city != '' and city.upper() != event.city.upper():
                    return False
                if categories and len(categories) > 0 and event.category not in categories:
                    return False
                return True

            def is_event_in_valid_period(self, event_date, date_from, date_to):
                if event_date >= date_from:
                    if not date_to:
                        return True
                    else:
                        if event_date <= date_to:
                            return True
                return False

            obj_id = Regexp(Link('./p/strong/a[@itemprop="url"]'), 'http://razibus.net/(.*).html')
            obj_summary = CleanText('./p/strong/a[@itemprop="url"]')
            obj_start_date = DateTime(CleanText('./p/span[@itemprop="startDate"]/@content'))
            obj_end_date = CombineDate(DateTime(CleanText('./p/span[@itemprop="startDate"]/@content')), EndTime('.'))
            obj_location = CleanText('./p/span[@itemprop="location"]/@content')
            obj_city = CleanText('./p/span[@itemprop="location"]')


class EventPage(HTMLPage):
    @method
    class get_event(ItemElement):
        klass = RazibusCalendarEvent

        obj_id = Env('_id')
        obj_summary = CleanText('//h2[@itemprop="name"]')
        obj_start_date = DateTime(CleanText('//span[@itemprop="startDate"]/time/@datetime'))
        obj_end_date = CombineDate(DateTime(CleanText('//span[@itemprop="startDate"]/time/@datetime')), EndTime('.'))
        obj_location = CleanText('//meta[@property="og:street-address"]/@content')
        obj_city = CleanText('//meta[@property="og:locality"]/@content')
        obj_url = CleanText('//meta[@property="og:url"]/@content')
        obj_description = CleanHTML('//div[@itemprop="description"]')



# -*- coding: utf-8 -*-

# Copyright(C) 2014      Roger Philibert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import datetime
from dateutil.parser import parse as parse_date
from dateutil.relativedelta import relativedelta
from dateutil.tz import tzlocal

from weboob.capabilities.messages import CapMessages, CapMessagesPost, Thread, Message
from weboob.capabilities.dating import CapDating, Optimization
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value, ValueBackendPassword
from weboob.tools.log import getLogger

from .browser import TinderBrowser, FacebookBrowser


__all__ = ['TinderModule']


class ProfilesWalker(Optimization):
    def __init__(self, sched, storage, browser):
        self._sched = sched
        self._storage = storage
        self._browser = browser
        self._logger = getLogger('walker', browser.logger)

        self._view_cron = None

    def start(self):
        self._view_cron = self._sched.schedule(1, self.view_profile)
        return True

    def stop(self):
        self._sched.cancel(self._view_cron)
        self._view_cron = None
        return True

    def set_config(self, params):
        pass

    def is_running(self):
        return self._view_cron is not None

    def view_profile(self):
        next_try = 1
        try:
            next_try = self._browser.like_profile()

            for thread in self._browser.get_threads():
                if 'person' not in thread:
                    continue

                other_name = thread['person']['name']
                if len(thread['messages']) == 0 and \
                   parse_date(thread['created_date']) < (datetime.datetime.now(tzlocal()) - relativedelta(hours=1)):
                    self._browser.post_message(thread['_id'], u'Coucou %s :)' % other_name)
                    self._logger.info(u'Welcome message sent to %s' % other_name)

        finally:
            if self._view_cron is not None:
                self._view_cron = self._sched.schedule(next_try, self.view_profile)


class TinderModule(Module, CapMessages, CapMessagesPost, CapDating):
    NAME = 'tinder'
    DESCRIPTION = u'Tinder dating mobile application'
    MAINTAINER = u'Roger Philibert'
    EMAIL = 'roger.philibert@gmail.com'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'
    CONFIG = BackendConfig(Value('username',                label='Facebook email'),
                           ValueBackendPassword('password', label='Facebook password'),
                           Value('location',                label='Location (example: 49.6008457,6.129709)', default=''))

    BROWSER = TinderBrowser
    STORAGE = {'contacts': {},
              }

    def create_default_browser(self):
        facebook = self.create_browser(klass=FacebookBrowser)
        facebook.login(self.config['username'].get(),
                       self.config['password'].get())
        return self.create_browser(facebook, self.config['location'].get())

    # ---- CapDating methods -----------------------

    def init_optimizations(self):
        self.add_optimization('PROFILE_WALKER', ProfilesWalker(self.weboob.scheduler, self.storage, self.browser))

    # ---- CapMessages methods ---------------------

    def fill_thread(self, thread, fields):
        return self.get_thread(thread)

    def iter_threads(self):
        for thread in self.browser.get_threads():
            if 'person' not in thread:
                # The account has been removed, probably because it was a
                # spammer.
                continue

            t = Thread(thread['_id'])
            t.flags = Thread.IS_DISCUSSION
            t.title = u'Discussion with %s' % thread['person']['name']
            contact = self.storage.get('contacts', t.id, default={'lastmsg': 0})

            birthday = parse_date(thread['person']['birth_date']).date()
            signature = u'Age: %d (%s)' % ((datetime.date.today() - birthday).days / 365.25, birthday)
            signature += u'\nLast ping: %s' % parse_date(thread['person']['ping_time']).strftime('%Y-%m-%d %H:%M:%S')
            signature += u'\nPhotos:\n\t%s' % '\n\t'.join([photo['url'] for photo in thread['person']['photos']])
            signature += u'\n\n%s' % thread['person'].get('bio', '')

            t.root = Message(thread=t, id=1, title=t.title,
                             sender=unicode(thread['person']['name']),
                             receivers=[self.browser.my_name],
                             date=parse_date(thread['created_date']),
                             content=u'Match!',
                             children=[],
                             signature=signature,
                             flags=Message.IS_UNREAD if int(contact['lastmsg']) < 1 else 0)
            parent = t.root

            for msg in thread['messages']:
                flags = 0
                if int(contact['lastmsg']) < msg['timestamp']:
                    flags = Message.IS_UNREAD

                msg = Message(thread=t,
                              id=msg['timestamp'],
                              title=t.title,
                              sender=unicode(self.browser.my_name if msg['from'] == self.browser.my_id else thread['person']['name']),
                              receivers=[unicode(self.browser.my_name if msg['to'] == self.browser.my_id else thread['person']['name'])],
                              date=parse_date(msg['sent_date']),
                              content=unicode(msg['message']),
                              children=[],
                              parent=parent,
                              signature=signature if msg['to'] == self.browser.my_id else u'',
                              flags=flags)
                parent.children.append(msg)
                parent = msg

            yield t

    def get_thread(self, _id):
        for t in self.iter_threads():
            if t.id == _id:
                return t

    def iter_unread_messages(self):
        for thread in self.iter_threads():
            for message in thread.iter_all_messages():
                if message.flags & message.IS_UNREAD:
                    yield message

    def set_message_read(self, message):
        contact = self.storage.get('contacts', message.thread.id, default={'lastmsg': 0})
        if int(contact['lastmsg']) < int(message.id):
            contact['lastmsg'] = int(message.id)
            self.storage.set('contacts', message.thread.id, contact)
            self.storage.save()

    # ---- CapMessagesPost methods ---------------------

    def post_message(self, message):
        self.browser.post_message(message.thread.id, message.content)

    OBJECTS = {Thread: fill_thread,
              }



# -*- coding: utf-8 -*-

# Copyright(C) 2015      Roger Philibert
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re

from weboob.browser.browsers import DomainBrowser
from weboob.browser.profiles import IPhone
from weboob.browser.pages import HTMLPage
from weboob.browser.filters.standard import CleanText
from weboob.exceptions import BrowserIncorrectPassword, ParseError
from weboob.tools.json import json


__all__ = ['HappnBrowser', 'FacebookBrowser']


class FacebookBrowser(DomainBrowser):
    BASEURL = 'https://graph.facebook.com'

    CLIENT_ID = "247294518656661"
    PROFILE = IPhone('Happn/3.0.2')
    access_token = None
    info = None

    def login(self, username, password):
        self.location('https://www.facebook.com/dialog/oauth?client_id=%s&redirect_uri=fbconnect://success&scope=email,user_birthday,user_friends,public_profile,user_photos,user_likes&response_type=token' % self.CLIENT_ID)
        page = HTMLPage(self, self.response)
        form = page.get_form('//form[@id="login_form"]')
        form['email'] = username
        form['pass'] = password
        form.submit(allow_redirects=False)
        if 'Location' not in self.response.headers:
            raise BrowserIncorrectPassword()

        self.location(self.response.headers['Location'])

        page = HTMLPage(self, self.response)
        if len(page.doc.xpath('//td/div[has-class("s")]')) > 0:
            raise BrowserIncorrectPassword(CleanText('//td/div[has-class("s")]')(page.doc))

        form = page.get_form(nr=0, submit='//input[@name="__CONFIRM__"]')
        form.submit()

        m = re.search('access_token=([^&]+)&', self.response.text)
        if m:
            self.access_token = m.group(1)
        else:
            raise ParseError('Unable to find access_token')

        self.info = self.request('/me')

    def request(self, url, *args, **kwargs):
        url += '&' if ('?' in url) else '?'
        url += 'access_token=' + self.access_token
        r = self.location(self.absurl(url, base=True), *args, **kwargs)
        return json.loads(r.content)


class HappnBrowser(DomainBrowser):
    BASEURL = 'https://api.happn.fr/'
    PROFILE = IPhone('Happn/18.3.1')
    ALLOW_REFERRER = False

    recs = []

    def __init__(self, facebook, *args, **kwargs):
        super(HappnBrowser, self).__init__(*args, **kwargs)
        self.facebook = facebook
        self.session.headers['User-Agent'] = 'Happn/18.3.1 AndroidSDK/11'

        r = self.request('/connect/oauth/token', data={
            'client_id': 'FUE-idSEP-f7AqCyuMcPr2K-1iCIU_YlvK-M-im3c',
            'client_secret': 'brGoHSwZsPjJ-lBk0HqEXVtb3UFu-y5l_JcOjD-Ekv',
            'grant_type': 'assertion',
            'assertion_type': 'facebook_access_token',
            'assertion': facebook.access_token,
            'scope': 'mobile_app',
        })
        self.session.headers['Authorization'] = 'OAuth="%s"' % r['access_token']

        self.my_id = r['user_id']
        self.refresh_token = r['refresh_token']

        self.request('/api/auth/proof', data={'facebook_access_token': facebook.access_token})

        r = self.request('/api/users/%s/devices/' % self.my_id,
                     data={"adid": "78462200ada313bb943b01800819a1d5",
                           "android_id": "fe26ca3be49bbbf6",
                           "app_build": "20.14.0",
                           "country_id": "FR",
                           "idfa": "2265540e-7a5f-4308-9c69-3afd2f608a2e",
                           "language_id": "fr",
                           "os_build": 23,
                           "token": "",
                           "type": "android"
                          })
        self.device_id = r['data']['id']

        me = self.request('/api/users/me')
        self.my_name = me['data']['name']

    def request(self, *args, **kwargs):
        r = self.location(*args, **kwargs)
        return r.json()

    def get_facebook(self, facebook_id):
        data = self.facebook.request('https://graph.facebook.com/%s' % facebook_id)
        if not 'link' in data:
            data['link'] = ('https://www.facebook.com/%s' % data['username']) if 'username' in data else ''
        likes = self.facebook.request('https://graph.facebook.com/%s/likes?limit=10000' % facebook_id)
        data['likes'] = [like['name'] for like in likes['data']]
        data['infos'] = self.facebook.request('https://graph.facebook.com/v2.3/%s?fields=about,address,age_range,bio,birthday,devices,education,email,favorite_athletes,favorite_teams,hometown,inspirational_people,install_type,interested_in,is_verified,languages,location,meeting_for,political,relationship_status,religion,significant_other,sports,quotes,timezone,updated_time,verified,website,work' % facebook_id)

        return data

    def get_contact(self, contact_id):
        data = self.request('/api/users/%s?fields=birth_date,first_name,fb_id,last_name,display_name,login,credits,referal,matching_preferences,notification_settings,unread_conversations,about,is_accepted,age,job,workplace,school,modification_date,profiles.mode(0).width(1000).height(1000).fields(url,width,height,mode),last_meet_position,my_relation,is_charmed,distance,gender' % contact_id)['data']
        if data['fb_id'] is not None:
            data['fb'] = self.get_facebook(data['fb_id'])
        return data

    def get_threads(self):
        return self.request('/api/users/me/conversations')['data']

    def get_thread(self, id):
        r = self.request('/api/users/%s/conversations/%s?fields=id,messages.limit(100).fields(id,message,creation_date,sender.fields(id)),participants.fields(user.fields(birth_date,first_name,last_name,display_name,credits,referal,matching_preferences,notification_settings,unread_conversations,about,is_accepted,age,job,workplace,school,modification_date,profiles.mode(0).width(1000).height(1000).fields(url,width,height,mode),last_meet_position,my_relation,is_charmed,distance,gender))' % (self.my_id, id))['data']
        return r

    def post_message(self, thread_id, content):
        self.request('/api/conversations/%s/messages/' % thread_id, data={'message': content})

    def find_users(self):
        return self.request('/api/users/me/notifications?types=468&fields=id,is_pushed,lon,actions,creation_date,is_notified,lat,modification_date,notification_type,nb_times,notifier.fields(id,job,is_accepted,workplace,my_relation,distance,gender,my_conversation,is_charmed,nb_photos,last_name,first_name,age),notified.fields(is_accepted,is_charmed)')['data']

    def accept(self, id):
        self.request('/api/users/me/accepted/%s' % id, method='POST')

    def set_position(self, lat, lng):
        r = self.request('/api/users/%s/devices/%s' % (self.my_id, self.device_id), method='PUT',
                         data={'latitude': lat, 'longitude': lng, 'altitude': 0.0})

        return r['data']['position']



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Bezleputh
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.
from io import BytesIO
import re
from decimal import Decimal
from weboob.browser.pages import HTMLPage, LoggedPage
from weboob.tools.captcha.virtkeyboard import MappedVirtKeyboard
from weboob.browser.elements import ItemElement, TableElement, method
from weboob.browser.filters.standard import CleanText, CleanDecimal, Format, Regexp, Date, Env, TableCell, Field
from weboob.browser.filters.html import CleanHTML
from weboob.capabilities.bank import Account, Transaction, Investment
from weboob.capabilities.base import NotAvailable


class VirtKeyboard(MappedVirtKeyboard):
    symbols = {'0': ('8adee734aaefb163fb008d26bb9b3a42', '922d79345bf824b1186d0aa523b37a7c'),
               '1': ('b815d6ce999910d48619b5912b81ddf1', '4730473dcd86f205dff51c59c97cf8c0'),
               '2': ('54255a70694787a4e1bd7dd473b50228', '2d8b1ab0b5ce0b88abbc0170d2e85b7e'),
               '3': ('ba06373d2bfba937d00bf52a31d475eb', '08e7e7ab7b330f3cfcb819b95eba64c6'),
               '4': ('3fa795ac70247922048c514115487b10', 'ffb3d035a3a335cfe32c59d8ee1302ad'),
               '5': ('788963d15fa05832ee7640f7c2a21bc3', 'c4b12545020cf87223901b6b35b9a9e2'),
               '6': ('c8bf62dfaed9feeb86934d8617182503', '473357666949855a0794f68f3fc40127'),
               '7': ('f7543fdda3039bdd383531954dd4fc46', '5f3a71bd2f696b8dc835dfeb7f32f92a'),
               '8': ('5c4210e2d8e39f7667d7a9e5534b18b7', 'b9a1a73430f724541108ed5dd862431b'),
               '9': ('94520ac801883fbfb700f43cd4172d41', '12c18ca3d4350acd077f557ac74161e5')}

    def __init__(self, page):
        self.img_id = page.doc.find("//input[@id='identifiantClavierVirtuel']").attrib['value']
        img = page.doc.find("//img[@id='clavier_virtuel']")
        res = page.browser.open('/portal/rest/clavier_virtuel/%s' % self.img_id)
        MappedVirtKeyboard.__init__(self, BytesIO(res.content), page.doc, img, (0, 0, 0), convert='RGB')

        self.check_symbols(self.symbols, page.browser.responses_dirname)

    def get_symbol_code(self, md5sum):
        code = MappedVirtKeyboard.get_symbol_code(self, md5sum)
        return ''.join(re.findall(r"'(\d+)'", code)[-2:])

    def get_string_code(self, string):
        code = ''
        for c in string:
            code += self.get_symbol_code(self.symbols[c])
        return code


class LoginPage(HTMLPage):
    pass


class HomePage(HTMLPage):
    def get_coded_passwd(self, password):
        vk = VirtKeyboard(self)
        return '%s|%s|#006#' % (vk.get_string_code(password), vk.img_id)

    def is_logged(self):
        return len(self.doc.find('//a[@class="btn-deconnexion"]'))


class AvoirPage(LoggedPage, HTMLPage):
    @method
    class get_account(ItemElement):
        klass = Account

        obj_label = Format('PEE %s', CleanText('//div[@id="pbGp_df83b8bd_2dd787_2d4d10_2db608_2d69c44af91e91_j_id1:j_idt1:j_idt2:j_idt15_body"]'))
        obj_balance = CleanDecimal('//div[@id="pbGp_3f1d2af7_2ddd41_2d45d2_2dbb9d_2d8f27b33a375f_j_id1:j_idt1:form:j_idt2:j_idt14:j_idt23:0:j_idt65:j_idt47_body"]',
                                   default=Decimal(0),
                                   replace_dots=True)
        obj_currency = Regexp(CleanText('//div[@id="pbGp_3f1d2af7_2ddd41_2d45d2_2dbb9d_2d8f27b33a375f_j_id1:j_idt1:form:j_idt2:j_idt14:j_idt23:0:j_idt65:j_idt47_body"]'),
                              '.*(.)$', default=u'€')
        obj_type = Account.TYPE_PEE

    @method
    class iter_investment(TableElement):
        head_xpath = u'//table[@id="pbGp_3f1d2af7_2ddd41_2d45d2_2dbb9d_2d8f27b33a375f_j_id1:j_idt1:form:j_idt2:j_idt413:j_idt447"]/thead/tr/th/@id'
        item_xpath = u'//table[@id="pbGp_3f1d2af7_2ddd41_2d45d2_2dbb9d_2d8f27b33a375f_j_id1:j_idt1:form:j_idt2:j_idt413:j_idt447"]/tbody/tr[@id]'

        col_reference = u'pbGp_3f1d2af7_2ddd41_2d45d2_2dbb9d_2d8f27b33a375f_j_id1:j_idt1:form:j_idt2:j_idt413:j_idt447:j_idt450'
        col_montant = u'pbGp_3f1d2af7_2ddd41_2d45d2_2dbb9d_2d8f27b33a375f_j_id1:j_idt1:form:j_idt2:j_idt413:j_idt447:j_idt456'
        col_repartition = u'pbGp_3f1d2af7_2ddd41_2d45d2_2dbb9d_2d8f27b33a375f_j_id1:j_idt1:form:j_idt2:j_idt413:j_idt447:j_idt460'

        class item(ItemElement):
            klass = Investment

            obj_label = CleanText(Regexp(CleanHTML(TableCell('reference')),
                                         '(.*)\n\n'))

            obj_vdate = Date(Regexp(CleanHTML(TableCell('reference')),
                                    '(\d{2}/\d{2}/\d{4})'))

            obj_unitvalue = CleanDecimal(Regexp(CleanHTML(TableCell('reference')),
                                                '.*\n\n(.*)\n\n'),
                                         replace_dots=True)

            obj_description = CleanText(CleanHTML(TableCell('reference')))

            obj_portfolio_share = CleanDecimal(CleanHTML(TableCell('repartition')),
                                               replace_dots=True)

            obj_valuation = CleanDecimal(CleanHTML(TableCell('montant')),
                                         replace_dots=True)

            def obj_quantity(self):
                return Decimal(Field('valuation')(self)/Field('unitvalue')(self))


class HistoryPage(LoggedPage, HTMLPage):
    @method
    class get_history(TableElement):
        head_xpath = u'//table[has-class("operation-bloc-content-tableau")]/thead/tr/th/div/div/div/div/div/div/a/text()'
        item_xpath = u'//table[has-class("operation-bloc-content-tableau")]/tbody/tr[has-class("rf-dt-r")]'

        col_date = u'Date de création'
        col_reference = u'Référence'
        col_montant = u'Montant net'
        col_type = u'Type d’opération'

        class item(ItemElement):
            klass = Transaction

            obj_date = Date(CleanText(TableCell('date')), Env('date_guesser'))
            obj_type = Transaction.TYPE_UNKNOWN
            obj_id = CleanText(TableCell('reference'))
            obj_label = CleanText(TableCell('type'))
            obj_amount = CleanDecimal(CleanHTML(TableCell('montant')),
                                      replace_dots=True, default=NotAvailable)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Cedric Defortis
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from weboob.browser import PagesBrowser, URL
from .pages import WeatherPage, SearchCitiesPage

__all__ = ['MeteofranceBrowser']


class MeteofranceBrowser(PagesBrowser):
    BASEURL = 'http://www.meteofrance.com'
    cities = URL('mf3-rpc-portlet/rest/lieu/facet/previsions/search/(?P<pattern>.*)', SearchCitiesPage)
    weather = URL('previsions-meteo-france/(?P<city_name>.*)/(?P<city_id>.*)', WeatherPage)

    def iter_city_search(self, pattern):
        return self.cities.go(pattern=pattern).iter_cities()

    def iter_forecast(self, city):
        return self.weather.go(city_id=city.id, city_name=city.name).iter_forecast()

    def get_current(self, city):
        return self.weather.go(city_id=city.id, city_name=city.name).get_current()



# -*- coding: utf-8 -*-

# Copyright(C) 2016      Benjamin Bouvier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.bank import CapBank
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value, ValueBackendPassword

from .browser import Number26Browser

__all__ = ['Number26Module']


class Number26Module(Module, CapBank):
    NAME = 'n26'
    DESCRIPTION = u'Bank N26'
    MAINTAINER = u'Benjamin Bouvier'
    EMAIL = 'public@benj.me'
    LICENSE = 'AGPLv3+'
    VERSION = '1.3'

    BROWSER = Number26Browser

    CONFIG = BackendConfig(
                 Value('login', label='Email', regexp='.+'),
                 ValueBackendPassword('password', label='Password')
             )

    STORAGE = {'categories': {}}

    def get_categories(self):
        categories = self.storage.get("categories", None)
        if categories is None:
            categories = self.browser.get_categories()
            self.storage.set("categories", categories)
        return categories

    def create_default_browser(self):
        return self.create_browser(self.config['login'].get(), self.config['password'].get())

    def iter_accounts(self):
        return self.browser.get_accounts()

    def get_account(self, id):
        return self.browser.get_account(id)

    def iter_history(self, account):
        categories = self.get_categories()
        return self.browser.get_transactions(categories)

    def iter_coming(self, account):
        categories = self.get_categories()
        return self.browser.get_coming(categories)



# -*- coding: utf-8 -*-

# Copyright(C) 2013      Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import re
from dateutil.parser import parse as parse_date

from weboob.capabilities.parcel import Parcel, Event
from weboob.capabilities import NotAvailable
from weboob.deprecated.browser import Page


class IndexPage(Page):
    def track_package(self, _id):
        self.browser.select_form(predicate=lambda form: form.attrs.get('id', '') == 'suivreEnvoi')
        self.browser['chronoNumbers'] = _id.encode('utf-8')
        self.browser.submit()


class TrackPage(Page):
    def get_info(self, id):
        if len(self.document.xpath('//libelle[@nom="MSG_AUCUN_EVT"]')) > 0:
            return None

        p = Parcel(id)
        p.arrival = NotAvailable
        p.history = []

        for i, tr in enumerate(self.document.xpath('//table[@class="tabListeEnvois"]//tr')):
            tds = tr.findall('td')
            if len(tds) < 3:
                continue

            ev = Event(i)
            ev.location = unicode(tds[1].text) if tds[1].text else None
            ev.activity = unicode(tds[1].find('br').tail)
            if tds[-1].text is not None:
                ev.activity += ', ' + self.parser.tocleanstring(tds[-1])
            date = re.sub('[a-z]+', '', self.parser.tocleanstring(tds[0])).strip()
            date = re.sub('(\d+)/(\d+)/(\d+)', r'\3-\2-\1', date)
            ev.date = parse_date(date)
            p.history.append(ev)

        p.info = ' '.join([t.strip() for t in self.document.xpath('//div[@class="numeroColi2"]')[0].itertext()][1:])
        if u'Livraison effectuée' in p.history[0].activity:
            p.status = p.STATUS_ARRIVED
        elif u"en cours d'acheminement" in p.history[0].activity or \
             u"en cours de livraison" in p.history[0].activity or \
             u"Envoi entré dans le pays de destination" in p.history[0].activity:
            p.status = p.STATUS_IN_TRANSIT

        return p



# -*- coding: utf-8 -*-

# Copyright(C) 2011  Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict

from weboob.capabilities.video import CapVideo, BaseVideo
from weboob.capabilities.collection import CapCollection, CollectionNotFound
from weboob.tools.backend import Module, BackendConfig
from weboob.tools.value import Value
from .browser import DailymotionBrowser

import re

__all__ = ['DailymotionModule']


class DailymotionModule(Module, CapVideo, CapCollection):
    NAME = 'dailymotion'
    MAINTAINER = u'Romain Bignon'
    EMAIL = 'romain@weboob.org'
    VERSION = '1.3'
    DESCRIPTION = 'Dailymotion video streaming website'
    LICENSE = 'AGPLv3+'
    BROWSER = DailymotionBrowser

    resolution_choice = OrderedDict([(k, u'%s (%s)' % (v, k)) for k, v in sorted({
        u'480': u'480p',
        u'240': u'240p',
        u'380': u'380p',
        u'720': u'720p',
        u'1080': u'1080p'
    }.iteritems())])

    format_choice = [u'm3u8', u'mp4']

    CONFIG = BackendConfig(Value('resolution', label=u'Resolution', choices=resolution_choice),
                           Value('format', label=u'Format', choices=format_choice))

    SORTBY = ['relevance', 'rated', 'visited', None]

    def create_default_browser(self):
        resolution = self.config['resolution'].get()
        format = self.config['format'].get()
        return self.create_browser(resolution=resolution, format=format)

    def get_video(self, _id):
        m = re.match('http://[w\.]*dailymotion\.com/video/(.*)', _id)
        if m:
            _id = m.group(1)

        if not _id.startswith('http'):
            return self.browser.get_video(_id)

    def search_videos(self, pattern, sortby=CapVideo.SEARCH_RELEVANCE, nsfw=False):
        return self.browser.search_videos(pattern, self.SORTBY[sortby])

    def fill_video(self, video, fields):
        if fields != ['thumbnail']:
            # if we don't want only the thumbnail, we probably want also every fields
            video = self.browser.get_video(video.id, video)
        if 'thumbnail' in fields and video.thumbnail:
            video.thumbnail.data = self.browser.open(video.thumbnail.url).content
        return video

    def iter_resources(self, objs, split_path):
        if BaseVideo in objs:
            collection = self.get_collection(objs, split_path)
            if collection.path_level == 0:
                yield self.get_collection(objs, [u'latest'])
            if collection.split_path == [u'latest']:
                for video in self.browser.latest_videos():
                    yield video

    def validate_collection(self, objs, collection):
        if collection.path_level == 0:
            return
        if BaseVideo in objs and collection.split_path == [u'latest']:
            collection.title = u'Latest Dailymotion videos'
            return
        raise CollectionNotFound(collection.split_path)

    OBJECTS = {BaseVideo: fill_video}



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011  Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.test import BackendTest
from weboob.tools.value import Value
from weboob.capabilities.video import BaseVideo

from random import choice
import itertools


class DailymotionTest(BackendTest):
    MODULE = 'dailymotion'

    # Not easy to find a kids video which will always be there
    # This might break in the future
    KIDS_VIDEO_TITLE = 'Telmo et Tula'

    def setUp(self):
        if not self.is_backend_configured():
            self.backend.config['resolution'] = Value(value='240')
            self.backend.config['format'] = Value(value='mp4')

    def test_search(self):
        l = list(itertools.islice(self.backend.search_videos('chirac'), 0, 20))
        self.assertTrue(len(l) > 0)
        v = choice(l)
        self.backend.fillobj(v, ('url',))
        self.assertTrue(v.url and v.url.startswith('http://'), 'URL for video "%s" not found: %s' % (v.id, v.url))

    def test_latest(self):
        l = list(itertools.islice(self.backend.iter_resources([BaseVideo], [u'latest']), 0, 20))
        assert len(l)
        v = choice(l)
        self.backend.fillobj(v, ('url',))
        self.assertTrue(v.url and v.url.startswith('http://'), 'URL for video "%s" not found: %s' % (v.id, v.url))

    def test_kids_video(self):
        l = list(itertools.islice(self.backend.search_videos(DailymotionTest.KIDS_VIDEO_TITLE), 0, 20))
        self.assertTrue(len(l) > 0)
        for elt in l[:10]:
            video_id = elt.id
            video = self.backend.get_video(video_id)
            self.assertIsNotNone(video.title)
            if DailymotionTest.KIDS_VIDEO_TITLE in video.title:
                self.assertTrue(video.url and video.url.startswith('http://'), 'URL for video "%s" not found: %s' %
                                (video.id, video.url))
                return

        self.fail("Can't find test video '%s' in kids.dailymotion.com video "
                  "on dailymotion, maybe the test video should be changed."
                  % DailymotionTest.KIDS_VIDEO_TITLE)



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011  Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

import lxml.html

from weboob.browser.filters.standard import CleanText

from .index import DLFPPage


class WikiEditPage(DLFPPage):
    def get_body(self):
        return CleanText('//textarea[has-class("wiki_page_wiki_body")]', default='')(self)

    form_xpath = '//form[@class="new_wiki_page" or @class="edit_wiki_page"]'

    def post_content(self, title, body, message):
        form = self.get_form(xpath=self.form_xpath)

        if title is not None:
            form['wiki_page[title]'] = title.encode('utf-8')
            form['commit'] = 'Créer'
        else:
            form['commit'] = 'Mettre à jour'
        form['wiki_page[wiki_body]'] = body.encode('utf-8')
        if message is not None:
            form['wiki_page[message]'] = message.encode('utf-8')

        form.submit()

    def post_preview(self, body):
        form = self.get_form(xpath=self.form_xpath)
        form['wiki_page[wiki_body]'] = body
        form.submit()

    def get_preview_html(self):
        body = self.doc.xpath('//article[has-class("wikipage")]//div[has-class("content")]')[0]
        return lxml.html.tostring(body)



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.deprecated.browser import Browser, BrowserHTTPNotFound

from .pages import SearchPage, SubtitlePage, LANGUAGE_NUMBERS


__all__ = ['PodnapisiBrowser']


class PodnapisiBrowser(Browser):
    DOMAIN = 'www.podnapisi.net'
    PROTOCOL = 'http'
    ENCODING = 'utf-8'
    USER_AGENT = Browser.USER_AGENTS['wget']
    PAGES = {
        'http://www.podnapisi.net/fr/ppodnapisi/search\?sJ=[0-9]*&sK=.*&sS=downloads&sO=desc': SearchPage,
        'http://www.podnapisi.net/fr/ppodnapisi/podnapis/i/[0-9]*': SubtitlePage
    }

    def iter_subtitles(self, language, pattern):
        nlang = LANGUAGE_NUMBERS[language]
        self.location('http://www.podnapisi.net/fr/ppodnapisi/search?sJ=%s&sK=%s&sS=downloads&sO=desc' % (nlang, pattern.encode('utf-8')))
        assert self.is_on_page(SearchPage)
        return self.page.iter_subtitles(unicode(language))

    def get_subtitle(self, id):
        try:
            self.location('http://www.podnapisi.net/fr/ppodnapisi/podnapis/i/%s' % id)
        except BrowserHTTPNotFound:
            return
        if self.is_on_page(SubtitlePage):
            return self.page.get_subtitle(id)



# -*- coding: utf-8 -*-

# Copyright(C) 2013      franek
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.deprecated.browser import Browser, BrowserIncorrectPassword
from weboob.deprecated.browser.decorators import id2url

from .pages import VideoPage, IndexPage, LoginPage, LoginRedirectPage
from .video import ArretSurImagesVideo


__all__ = ['ArretSurImagesBrowser']


class ArretSurImagesBrowser(Browser):
    PROTOCOL = 'http'
    DOMAIN = 'www.arretsurimages.net'
    ENCODING = None

    PAGES = {
        '%s://%s/contenu.php\?id=.+' % (PROTOCOL, DOMAIN): VideoPage,
        '%s://%s/emissions.php' % (PROTOCOL, DOMAIN): IndexPage,
        '%s://%s/forum/login.php' % (PROTOCOL, DOMAIN): LoginPage,
        '%s://%s/forum/index.php' % (PROTOCOL, DOMAIN): LoginRedirectPage,
    }

    def home(self):
        self.location('http://www.arretsurimages.net')

    def search_videos(self, pattern):
        self.location(self.buildurl('/emissions.php'))
        assert self.is_on_page(IndexPage)
        return self.page.iter_videos(pattern)

    @id2url(ArretSurImagesVideo.id2url)
    def get_video(self, url, video=None):
        self.login()
        self.location(url)
        return self.page.get_video(video)

    def is_logged(self):
        return not self.is_on_page(LoginPage)

    def login(self):
        if not self.is_on_page(LoginPage):
            self.location('http://www.arretsurimages.net/forum/login.php', no_login=True)

        self.page.login(self.username, self.password)

        if not self.is_logged():
            raise BrowserIncorrectPassword()

    def latest_videos(self):
        self.location(self.buildurl('/emissions.php'))
        assert self.is_on_page(IndexPage)
        return self.page.iter_videos()



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2014 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.capabilities.radio import CapRadio, Radio
from weboob.capabilities.audiostream import BaseAudioStream
from weboob.tools.capabilities.streaminfo import StreamInfo
from weboob.capabilities.collection import CapCollection
from weboob.tools.backend import Module
from weboob.browser.browsers import APIBrowser
from weboob.tools.misc import to_unicode


__all__ = ['OuiFMModule']


class OuiFMModule(Module, CapRadio, CapCollection):
    NAME = 'ouifm'
    MAINTAINER = u'Romain Bignon'
    EMAIL = 'romain@weboob.org'
    VERSION = '1.3'
    DESCRIPTION = u'OÜI FM French radio'
    LICENSE = 'AGPLv3+'
    BROWSER = APIBrowser

    _RADIOS = {'general':     (u"OÜI FM",
                               u'OÜI FM',
                               u'http://stream.ouifm.fr/ouifm-high.mp3"', 160),
               'alternatif':  (u"OÜI FM Alternatif",
                               u'OÜI FM - Alternatif',
                               u'http://alternatif.stream.ouifm.fr/ouifm2.mp3', 128),
               'classicrock': (u"OÜI FM Classic Rock",
                               u'OÜI FM - Classic Rock',
                               u'http://classicrock.stream.ouifm.fr/ouifm3.mp3', 128),
               'bluesnrock':  (u"OÜI FM Blues'n'Rock",
                               u'OÜI FM - Blues\'n\'Rock',
                               u'http://bluesnrock.stream.ouifm.fr/ouifmbluesnrock-128.mp3', 128),
               'rockinde':    (u"OÜI FM Rock Indé",
                               u'OÜI FM - Rock Indé',
                               u'http://rockinde.stream.ouifm.fr/ouifm5.mp3', 128),
               'ganja':       (u"OÜI FM Ganja",
                               u'OÜI FM - Ganja',
                               u'http://ganja.stream.ouifm.fr/ouifmganja-128.mp3', 128),
               'rock60s':     (u"OÜI FM Rock 60's",
                               u'OÜI FM - Rock 60\'s',
                               u'http://rock60s.stream.ouifm.fr/ouifmsixties.mp3', 128),
               'rock70s':     (u"OÜI FM Rock 70's",
                               u'OÜI FM - Rock 70\'s',
                               u'http://rock70s.stream.ouifm.fr/ouifmseventies.mp3', 128),
               'rock80s':     (u"OÜI FM Rock 80's",
                               u'OÜI FM - Rock 80\'s',
                               u'http://rock80s.stream.ouifm.fr/ouifmeighties.mp3', 128),
               'rock90s':     (u"OÜI FM Rock 90's",
                               u'OÜI FM - Rock 90\'s',
                               u'http://rock90s.stream.ouifm.fr/ouifmnineties.mp3', 128),
               'rock2000':    (u"OÜI FM Rock 2000",
                               u'OÜI FM - Rock 2000',
                               u'http://rock2000.stream.ouifm.fr/ouifmrock2000.mp3', 128),
               'summertime':  (u"OÜI FM Summertime",
                               u'OÜI FM - Summertime',
                               u'http://summertime.stream.ouifm.fr/ouifmsummertime.mp3', 128)}

    def iter_resources(self, objs, split_path):
        if Radio in objs:
            self._restrict_level(split_path)

            for id in self._RADIOS:
                yield self.get_radio(id)

    def iter_radios_search(self, pattern):
        for radio in self.iter_resources((Radio, ), []):
            if pattern.lower() in radio.title.lower() or pattern.lower() in radio.description.lower():
                yield radio

    def get_current(self, radio):
        document = self.browser.request('http://www.ouifm.fr/onair.json')
        rad = ''
        if radio == 'general':
            rad = 'rock'
        else:
            rad = radio

        last = document[rad][0]

        artist = to_unicode(last.get('artist', '').strip())
        title = to_unicode(last.get('title', '').strip())
        return artist, title

    def get_radio(self, radio):
        if not isinstance(radio, Radio):
            radio = Radio(radio)

        if radio.id not in self._RADIOS:
            return None

        title, description, url, bitrate = self._RADIOS[radio.id]
        radio.title = title
        radio.description = description

        artist, title = self.get_current(radio.id)
        current = StreamInfo(0)
        current.who = artist
        current.what = title
        radio.current = current

        stream = BaseAudioStream(0)
        stream.bitrate = bitrate
        stream.format = u'mp3'
        stream.title = u'%skbits/s' % (stream.bitrate)
        stream.url = url
        radio.streams = [stream]
        return radio

    def fill_radio(self, radio, fields):
        if 'current' in fields:
            if not radio.current:
                radio.current = StreamInfo(0)
            radio.current.who, radio.current.what = self.get_current(radio.id)
        return radio

    OBJECTS = {Radio: fill_radio}



# -*- coding: utf-8 -*-

# Copyright(C) 2010-2011 Christophe Benz
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.compat import quote_plus

from .pages.compose import ClosePage, ComposePage, ConfirmPage, SentPage
from .pages.login import LoginPage

from weboob.deprecated.browser import Browser, BrowserIncorrectPassword


__all__ = ['SfrBrowser']


class SfrBrowser(Browser):
    DOMAIN = 'www.sfr.fr'
    PAGES = {
        'http://messagerie-.+.sfr.fr/webmail/close_xms_tab.html': ClosePage,
        'http://www.sfr.fr/xmscomposer/index.html\?todo=compose': ComposePage,
        'http://www.sfr.fr/xmscomposer/mc/envoyer-texto-mms/confirm.html': ConfirmPage,
        'https://www.sfr.fr/cas/login\?service=.*': LoginPage,
        'http://www.sfr.fr/xmscomposer/mc/envoyer-texto-mms/send.html': SentPage,
        }

    def get_nb_remaining_free_sms(self):
        if not self.is_on_page(ComposePage):
            self.home()
        return self.page.get_nb_remaining_free_sms()

    def home(self):
        self.location('http://www.sfr.fr/xmscomposer/index.html?todo=compose')

    def is_logged(self):
        return 'loginForm' not in [form.name for form in self.forms()]

    def login(self):
        service_url = 'http://www.sfr.fr/xmscomposer/j_spring_cas_security_check'
        self.location('https://www.sfr.fr/cas/login?service=%s' % quote_plus(service_url), no_login=True)
        self.page.login(self.username, self.password)
        if not self.is_logged():
            raise BrowserIncorrectPassword()

    def post_message(self, message):
        if not self.is_on_page(ComposePage):
            self.home()
        self.page.post_message(message)
        if self.is_on_page(ConfirmPage):
            self.page.confirm()
        assert self.is_on_page(ClosePage) or self.is_on_page(SentPage)



# -*- coding: utf-8 -*-

# Copyright(C) 2016      James GALT
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
from decimal import Decimal

from weboob.browser.elements import ItemElement, method, DictElement
from weboob.browser.filters.standard import CleanDecimal, Date, Field, CleanText, Env
from weboob.browser.filters.json import Dict
from weboob.browser.pages import LoggedPage, JsonPage
from weboob.capabilities.bank import Account, Investment, Transaction
from weboob.capabilities.base import NotAvailable
from weboob.exceptions import NoAccountsException


class LoginPage(JsonPage):
    def get_token(self):
        return Dict('token')(self.doc)


class AccountsPage(LoggedPage, JsonPage):
    ACCOUNT_TYPES = {'PEE': Account.TYPE_PEE,
                     'PEG': Account.TYPE_PEE,
                     'PERCO': Account.TYPE_PERCO,
                     'RSP': Account.TYPE_RSP
                    }

    @method
    class iter_accounts(DictElement):
        def parse(self, el):
            if not el.get('count', 42):
                raise NoAccountsException()

        item_xpath = "listPositionsSalarieFondsDto/*/positionsSalarieDispositifDto"

        class item(ItemElement):
            klass = Account

            obj_id = CleanText(Dict('codeDispositif'))
            obj_balance = CleanDecimal(Dict('mtBrut'))
            obj_number = Field('id')
            obj_currency = u"EUR"

            def obj_type(self):
                return self.page.ACCOUNT_TYPES.get(Dict('typeDispositif')(self), Account.TYPE_LIFE_INSURANCE)

            def obj_label(self):
                try:
                    return Dict('libelleDispositif')(self).encode('iso-8859-2').decode('utf8')
                except (UnicodeEncodeError, UnicodeDecodeError):
                    try:
                        return Dict('libelleDispositif')(self).encode('latin1').decode('utf8')
                    except UnicodeDecodeError:
                        return Dict('libelleDispositif')(self)

    @method
    class iter_investments(DictElement):
        def find_elements(self):
            for psds in Dict('listPositionsSalarieFondsDto')(self):
                for psd in psds.get('positionsSalarieDispositifDto'):
                    if psd.get('codeDispositif') == Env('account_id')(self):
                        return psd.get('positionsSalarieFondsDto')
            return {}

        class item(ItemElement):
            klass = Investment

            obj_label = Dict('libelleFonds')
            obj_unitvalue = Dict('vl') & CleanDecimal
            obj_quantity = Dict('nbParts') & CleanDecimal
            obj_valuation = Dict('mtBrut') & CleanDecimal
            obj_code = Dict('codeIsin', default=NotAvailable)
            obj_vdate = Date(Dict('dtVl'))
            obj_diff = Dict('mtPMV') & CleanDecimal


class AccountHistoryPage(LoggedPage, JsonPage):
    def belongs(self, instructions, account):
        for ins in instructions:
            if 'nomDispositif' in ins and 'codeDispositif' in ins and '%s%s' % (ins['nomDispositif'], ins['codeDispositif']) == \
               '%s%s' % (account.label, account.id):
                return True
        return False

    def get_amount(self, instructions, account):
        amount = 0

        for ins in instructions:
            if 'nomDispositif' in ins and 'montantNet' in ins and 'codeDispositif' in ins and '%s%s' % (ins['nomDispositif'], ins['codeDispositif']) == \
               '%s%s' % (account.label, account.id):
                amount += ins['montantNet']

        return Decimal(amount)

    def iter_history(self, account):
        for hist in self.doc['operationsIndividuelles']:
            if len(hist['instructions']) > 0:
                if self.belongs(hist['instructions'], account):
                    tr = Transaction()
                    tr.amount = self.get_amount(hist['instructions'], account)
                    tr.rdate = datetime.strptime(hist['dateComptabilisation'].split('T')[0], '%Y-%m-%d')
                    tr.date = tr.rdate
                    tr.label = hist['libelleOperation'] if 'libelleOperation' in hist else hist['libelleCommunication']
                    tr.type = Transaction.TYPE_UNKNOWN

                    # Bypassed because we don't have the ISIN code
                    # tr.investments = []
                    # for ins in hist['instructions']:
                    #     inv = Investment()
                    #     inv.code = NotAvailable
                    #     inv.label = ins['nomFonds']
                    #     inv.description = ' '.join([ins['type'], ins['nomDispositif']])
                    #     inv.vdate = datetime.strptime(ins.get('dateVlReel', ins.get('dateVlExecution')).split('T')[
                    # 0], '%Y-%m-%d')
                    #     inv.valuation = Decimal(ins['montantNet'])
                    #     inv.quantity = Decimal(ins['nombreDeParts'])
                    #     inv.unitprice = inv.unitvalue = Decimal(ins['vlReel'])
                    #     tr.investments.append(inv)

                    yield tr



# -*- coding: utf-8 -*-

# Copyright(C) 2013 Julien Veyssier
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.deprecated.browser import Browser, BrowserHTTPNotFound

from .pages import TorrentsPage, TorrentPage


__all__ = ['BtmonBrowser']


class BtmonBrowser(Browser):
    DOMAIN = 'www.btmon.com'
    PROTOCOL = 'http'
    ENCODING = 'utf-8'
    USER_AGENT = Browser.USER_AGENTS['wget']
    PAGES = {
        'http://www.btmon.com/torrent/\?f=.*': TorrentsPage,
        'http://www.btmon.com/.*torrent.html': TorrentPage,
        }

    def iter_torrents(self, pattern):
        self.location('http://www.btmon.com/torrent/?f=%s' % pattern.encode('utf-8'))
        assert self.is_on_page(TorrentsPage)
        return self.page.iter_torrents()

    def get_torrent(self, id):
        try:
            self.location('http://www.btmon.com/%s.html' % id)
        except BrowserHTTPNotFound:
            return
        if self.is_on_page(TorrentPage):
            return self.page.get_torrent()



# -*- coding: utf-8 -*-

# Copyright(C) 2015 Guilhem Bonnefille
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


from weboob.tools.test import BackendTest
from weboob.capabilities.video import BaseVideo


class RmllTest(BackendTest):
    MODULE = 'rmll'

    def test_video_search(self):
        videos = self.backend.search_videos('test')
        self.assertTrue(videos)
        for video in videos:
            self.assertTrue(video.id, 'ID for video not found')

    def test_video_page(self):
        for slug in ["v124f0bc409e704d92cf", "%s/permalink/v124f0bc409e704d92cf/" % self.backend.browser.BASEURL]:
            video = self.backend.browser.get_video(slug)
            self.assertTrue(video.id, 'ID for video not found')
            self.assertTrue(video.url, 'URL for video "%s" not found' % (video.id))
            self.assertTrue(video.thumbnail, 'Thumbnail for video "%s" not found' % (video.id))
            self.assertTrue(video.title, 'Title for video "%s" not found' % (video.id))
            # self.assertTrue(video.description, 'Description for video "%s" not found' % (video.id))
            self.assertTrue(video.duration, 'Duration for video "%s" not found' % (video.id))
            # help(video)

    def test_video_fill(self):
        slug = "v124f0bc409e704d92cf"
        video = self.backend.browser.get_video(slug)
        video = self.backend.fill_video(video, ["url"])
        self.assertTrue(video)
        self.assertTrue(video.url, 'URL for video "%s" not found' % (video.id))

    def test_browse(self):
        for path in [[], ['latest']]:
            videos = self.backend.iter_resources([BaseVideo], path)
            self.assertTrue(videos)
            for video in videos:
                self.assertTrue(video.id, 'ID for video not found')

    def test_missing_duration(self):
        videos = self.backend.search_videos('weboob')
        self.assertTrue(videos)
        for video in videos:
            self.assertTrue(video.id, 'ID for video not found')
            video = self.backend.fill_video(video, ["$full"])
